# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/hyperparams.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/hyperparams.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n9rastervision/protos/tf_object_detection/hyperparams.proto\x12\'rastervision.protos.tf_object_detection\"\xfc\x03\n\x0bHyperparams\x12I\n\x02op\x18\x01 \x01(\x0e\x32\x37.rastervision.protos.tf_object_detection.Hyperparams.Op:\x04\x43ONV\x12I\n\x0bregularizer\x18\x02 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Regularizer\x12I\n\x0binitializer\x18\x03 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Initializer\x12Y\n\nactivation\x18\x04 \x01(\x0e\x32?.rastervision.protos.tf_object_detection.Hyperparams.Activation:\x04RELU\x12\x46\n\nbatch_norm\x18\x05 \x01(\x0b\x32\x32.rastervision.protos.tf_object_detection.BatchNorm\x12#\n\x14regularize_depthwise\x18\x06 \x01(\x08:\x05\x66\x61lse\"\x16\n\x02Op\x12\x08\n\x04\x43ONV\x10\x01\x12\x06\n\x02\x46\x43\x10\x02\",\n\nActivation\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04RELU\x10\x01\x12\n\n\x06RELU_6\x10\x02\"\xc6\x01\n\x0bRegularizer\x12P\n\x0el1_regularizer\x18\x01 \x01(\x0b\x32\x36.rastervision.protos.tf_object_detection.L1RegularizerH\x00\x12P\n\x0el2_regularizer\x18\x02 \x01(\x0b\x32\x36.rastervision.protos.tf_object_detection.L2RegularizerH\x00\x42\x13\n\x11regularizer_oneof\"\"\n\rL1Regularizer\x12\x11\n\x06weight\x18\x01 \x01(\x02:\x01\x31\"\"\n\rL2Regularizer\x12\x11\n\x06weight\x18\x01 \x01(\x02:\x01\x31\"\xe3\x02\n\x0bInitializer\x12k\n\x1ctruncated_normal_initializer\x18\x01 \x01(\x0b\x32\x43.rastervision.protos.tf_object_detection.TruncatedNormalInitializerH\x00\x12k\n\x1cvariance_scaling_initializer\x18\x02 \x01(\x0b\x32\x43.rastervision.protos.tf_object_detection.VarianceScalingInitializerH\x00\x12\x65\n\x19random_normal_initializer\x18\x03 \x01(\x0b\x32@.rastervision.protos.tf_object_detection.RandomNormalInitializerH\x00\x42\x13\n\x11initializer_oneof\"@\n\x1aTruncatedNormalInitializer\x12\x0f\n\x04mean\x18\x01 \x01(\x02:\x01\x30\x12\x11\n\x06stddev\x18\x02 \x01(\x02:\x01\x31\"\xd5\x01\n\x1aVarianceScalingInitializer\x12\x11\n\x06\x66\x61\x63tor\x18\x01 \x01(\x02:\x01\x32\x12\x16\n\x07uniform\x18\x02 \x01(\x08:\x05\x66\x61lse\x12^\n\x04mode\x18\x03 \x01(\x0e\x32H.rastervision.protos.tf_object_detection.VarianceScalingInitializer.Mode:\x06\x46\x41N_IN\",\n\x04Mode\x12\n\n\x06\x46\x41N_IN\x10\x00\x12\x0b\n\x07\x46\x41N_OUT\x10\x01\x12\x0b\n\x07\x46\x41N_AVG\x10\x02\"=\n\x17RandomNormalInitializer\x12\x0f\n\x04mean\x18\x01 \x01(\x02:\x01\x30\x12\x11\n\x06stddev\x18\x02 \x01(\x02:\x01\x31\"z\n\tBatchNorm\x12\x14\n\x05\x64\x65\x63\x61y\x18\x01 \x01(\x02:\x05\x30.999\x12\x14\n\x06\x63\x65nter\x18\x02 \x01(\x08:\x04true\x12\x14\n\x05scale\x18\x03 \x01(\x08:\x05\x66\x61lse\x12\x16\n\x07\x65psilon\x18\x04 \x01(\x02:\x05\x30.001\x12\x13\n\x05train\x18\x05 \x01(\x08:\x04true')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_HYPERPARAMS_OP = _descriptor.EnumDescriptor(
  name='Op',
  full_name='rastervision.protos.tf_object_detection.Hyperparams.Op',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONV', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FC', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=543,
  serialized_end=565,
)
_sym_db.RegisterEnumDescriptor(_HYPERPARAMS_OP)

_HYPERPARAMS_ACTIVATION = _descriptor.EnumDescriptor(
  name='Activation',
  full_name='rastervision.protos.tf_object_detection.Hyperparams.Activation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELU', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RELU_6', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=567,
  serialized_end=611,
)
_sym_db.RegisterEnumDescriptor(_HYPERPARAMS_ACTIVATION)

_VARIANCESCALINGINITIALIZER_MODE = _descriptor.EnumDescriptor(
  name='Mode',
  full_name='rastervision.protos.tf_object_detection.VarianceScalingInitializer.Mode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FAN_IN', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAN_OUT', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAN_AVG', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1480,
  serialized_end=1524,
)
_sym_db.RegisterEnumDescriptor(_VARIANCESCALINGINITIALIZER_MODE)


_HYPERPARAMS = _descriptor.Descriptor(
  name='Hyperparams',
  full_name='rastervision.protos.tf_object_detection.Hyperparams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='op', full_name='rastervision.protos.tf_object_detection.Hyperparams.op', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='regularizer', full_name='rastervision.protos.tf_object_detection.Hyperparams.regularizer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='initializer', full_name='rastervision.protos.tf_object_detection.Hyperparams.initializer', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='activation', full_name='rastervision.protos.tf_object_detection.Hyperparams.activation', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='batch_norm', full_name='rastervision.protos.tf_object_detection.Hyperparams.batch_norm', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='regularize_depthwise', full_name='rastervision.protos.tf_object_detection.Hyperparams.regularize_depthwise', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _HYPERPARAMS_OP,
    _HYPERPARAMS_ACTIVATION,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=611,
)


_REGULARIZER = _descriptor.Descriptor(
  name='Regularizer',
  full_name='rastervision.protos.tf_object_detection.Regularizer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='l1_regularizer', full_name='rastervision.protos.tf_object_detection.Regularizer.l1_regularizer', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='l2_regularizer', full_name='rastervision.protos.tf_object_detection.Regularizer.l2_regularizer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='regularizer_oneof', full_name='rastervision.protos.tf_object_detection.Regularizer.regularizer_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=614,
  serialized_end=812,
)


_L1REGULARIZER = _descriptor.Descriptor(
  name='L1Regularizer',
  full_name='rastervision.protos.tf_object_detection.L1Regularizer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='weight', full_name='rastervision.protos.tf_object_detection.L1Regularizer.weight', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=814,
  serialized_end=848,
)


_L2REGULARIZER = _descriptor.Descriptor(
  name='L2Regularizer',
  full_name='rastervision.protos.tf_object_detection.L2Regularizer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='weight', full_name='rastervision.protos.tf_object_detection.L2Regularizer.weight', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=850,
  serialized_end=884,
)


_INITIALIZER = _descriptor.Descriptor(
  name='Initializer',
  full_name='rastervision.protos.tf_object_detection.Initializer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='truncated_normal_initializer', full_name='rastervision.protos.tf_object_detection.Initializer.truncated_normal_initializer', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='variance_scaling_initializer', full_name='rastervision.protos.tf_object_detection.Initializer.variance_scaling_initializer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='random_normal_initializer', full_name='rastervision.protos.tf_object_detection.Initializer.random_normal_initializer', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='initializer_oneof', full_name='rastervision.protos.tf_object_detection.Initializer.initializer_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=887,
  serialized_end=1242,
)


_TRUNCATEDNORMALINITIALIZER = _descriptor.Descriptor(
  name='TruncatedNormalInitializer',
  full_name='rastervision.protos.tf_object_detection.TruncatedNormalInitializer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mean', full_name='rastervision.protos.tf_object_detection.TruncatedNormalInitializer.mean', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stddev', full_name='rastervision.protos.tf_object_detection.TruncatedNormalInitializer.stddev', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1244,
  serialized_end=1308,
)


_VARIANCESCALINGINITIALIZER = _descriptor.Descriptor(
  name='VarianceScalingInitializer',
  full_name='rastervision.protos.tf_object_detection.VarianceScalingInitializer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='factor', full_name='rastervision.protos.tf_object_detection.VarianceScalingInitializer.factor', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(2),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uniform', full_name='rastervision.protos.tf_object_detection.VarianceScalingInitializer.uniform', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mode', full_name='rastervision.protos.tf_object_detection.VarianceScalingInitializer.mode', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _VARIANCESCALINGINITIALIZER_MODE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1311,
  serialized_end=1524,
)


_RANDOMNORMALINITIALIZER = _descriptor.Descriptor(
  name='RandomNormalInitializer',
  full_name='rastervision.protos.tf_object_detection.RandomNormalInitializer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mean', full_name='rastervision.protos.tf_object_detection.RandomNormalInitializer.mean', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stddev', full_name='rastervision.protos.tf_object_detection.RandomNormalInitializer.stddev', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1526,
  serialized_end=1587,
)


_BATCHNORM = _descriptor.Descriptor(
  name='BatchNorm',
  full_name='rastervision.protos.tf_object_detection.BatchNorm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='decay', full_name='rastervision.protos.tf_object_detection.BatchNorm.decay', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.999),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='center', full_name='rastervision.protos.tf_object_detection.BatchNorm.center', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scale', full_name='rastervision.protos.tf_object_detection.BatchNorm.scale', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='epsilon', full_name='rastervision.protos.tf_object_detection.BatchNorm.epsilon', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.001),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train', full_name='rastervision.protos.tf_object_detection.BatchNorm.train', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1589,
  serialized_end=1711,
)

_HYPERPARAMS.fields_by_name['op'].enum_type = _HYPERPARAMS_OP
_HYPERPARAMS.fields_by_name['regularizer'].message_type = _REGULARIZER
_HYPERPARAMS.fields_by_name['initializer'].message_type = _INITIALIZER
_HYPERPARAMS.fields_by_name['activation'].enum_type = _HYPERPARAMS_ACTIVATION
_HYPERPARAMS.fields_by_name['batch_norm'].message_type = _BATCHNORM
_HYPERPARAMS_OP.containing_type = _HYPERPARAMS
_HYPERPARAMS_ACTIVATION.containing_type = _HYPERPARAMS
_REGULARIZER.fields_by_name['l1_regularizer'].message_type = _L1REGULARIZER
_REGULARIZER.fields_by_name['l2_regularizer'].message_type = _L2REGULARIZER
_REGULARIZER.oneofs_by_name['regularizer_oneof'].fields.append(
  _REGULARIZER.fields_by_name['l1_regularizer'])
_REGULARIZER.fields_by_name['l1_regularizer'].containing_oneof = _REGULARIZER.oneofs_by_name['regularizer_oneof']
_REGULARIZER.oneofs_by_name['regularizer_oneof'].fields.append(
  _REGULARIZER.fields_by_name['l2_regularizer'])
_REGULARIZER.fields_by_name['l2_regularizer'].containing_oneof = _REGULARIZER.oneofs_by_name['regularizer_oneof']
_INITIALIZER.fields_by_name['truncated_normal_initializer'].message_type = _TRUNCATEDNORMALINITIALIZER
_INITIALIZER.fields_by_name['variance_scaling_initializer'].message_type = _VARIANCESCALINGINITIALIZER
_INITIALIZER.fields_by_name['random_normal_initializer'].message_type = _RANDOMNORMALINITIALIZER
_INITIALIZER.oneofs_by_name['initializer_oneof'].fields.append(
  _INITIALIZER.fields_by_name['truncated_normal_initializer'])
_INITIALIZER.fields_by_name['truncated_normal_initializer'].containing_oneof = _INITIALIZER.oneofs_by_name['initializer_oneof']
_INITIALIZER.oneofs_by_name['initializer_oneof'].fields.append(
  _INITIALIZER.fields_by_name['variance_scaling_initializer'])
_INITIALIZER.fields_by_name['variance_scaling_initializer'].containing_oneof = _INITIALIZER.oneofs_by_name['initializer_oneof']
_INITIALIZER.oneofs_by_name['initializer_oneof'].fields.append(
  _INITIALIZER.fields_by_name['random_normal_initializer'])
_INITIALIZER.fields_by_name['random_normal_initializer'].containing_oneof = _INITIALIZER.oneofs_by_name['initializer_oneof']
_VARIANCESCALINGINITIALIZER.fields_by_name['mode'].enum_type = _VARIANCESCALINGINITIALIZER_MODE
_VARIANCESCALINGINITIALIZER_MODE.containing_type = _VARIANCESCALINGINITIALIZER
DESCRIPTOR.message_types_by_name['Hyperparams'] = _HYPERPARAMS
DESCRIPTOR.message_types_by_name['Regularizer'] = _REGULARIZER
DESCRIPTOR.message_types_by_name['L1Regularizer'] = _L1REGULARIZER
DESCRIPTOR.message_types_by_name['L2Regularizer'] = _L2REGULARIZER
DESCRIPTOR.message_types_by_name['Initializer'] = _INITIALIZER
DESCRIPTOR.message_types_by_name['TruncatedNormalInitializer'] = _TRUNCATEDNORMALINITIALIZER
DESCRIPTOR.message_types_by_name['VarianceScalingInitializer'] = _VARIANCESCALINGINITIALIZER
DESCRIPTOR.message_types_by_name['RandomNormalInitializer'] = _RANDOMNORMALINITIALIZER
DESCRIPTOR.message_types_by_name['BatchNorm'] = _BATCHNORM

Hyperparams = _reflection.GeneratedProtocolMessageType('Hyperparams', (_message.Message,), dict(
  DESCRIPTOR = _HYPERPARAMS,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.Hyperparams)
  ))
_sym_db.RegisterMessage(Hyperparams)

Regularizer = _reflection.GeneratedProtocolMessageType('Regularizer', (_message.Message,), dict(
  DESCRIPTOR = _REGULARIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.Regularizer)
  ))
_sym_db.RegisterMessage(Regularizer)

L1Regularizer = _reflection.GeneratedProtocolMessageType('L1Regularizer', (_message.Message,), dict(
  DESCRIPTOR = _L1REGULARIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.L1Regularizer)
  ))
_sym_db.RegisterMessage(L1Regularizer)

L2Regularizer = _reflection.GeneratedProtocolMessageType('L2Regularizer', (_message.Message,), dict(
  DESCRIPTOR = _L2REGULARIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.L2Regularizer)
  ))
_sym_db.RegisterMessage(L2Regularizer)

Initializer = _reflection.GeneratedProtocolMessageType('Initializer', (_message.Message,), dict(
  DESCRIPTOR = _INITIALIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.Initializer)
  ))
_sym_db.RegisterMessage(Initializer)

TruncatedNormalInitializer = _reflection.GeneratedProtocolMessageType('TruncatedNormalInitializer', (_message.Message,), dict(
  DESCRIPTOR = _TRUNCATEDNORMALINITIALIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.TruncatedNormalInitializer)
  ))
_sym_db.RegisterMessage(TruncatedNormalInitializer)

VarianceScalingInitializer = _reflection.GeneratedProtocolMessageType('VarianceScalingInitializer', (_message.Message,), dict(
  DESCRIPTOR = _VARIANCESCALINGINITIALIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.VarianceScalingInitializer)
  ))
_sym_db.RegisterMessage(VarianceScalingInitializer)

RandomNormalInitializer = _reflection.GeneratedProtocolMessageType('RandomNormalInitializer', (_message.Message,), dict(
  DESCRIPTOR = _RANDOMNORMALINITIALIZER,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.RandomNormalInitializer)
  ))
_sym_db.RegisterMessage(RandomNormalInitializer)

BatchNorm = _reflection.GeneratedProtocolMessageType('BatchNorm', (_message.Message,), dict(
  DESCRIPTOR = _BATCHNORM,
  __module__ = 'rastervision.protos.tf_object_detection.hyperparams_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.BatchNorm)
  ))
_sym_db.RegisterMessage(BatchNorm)


# @@protoc_insertion_point(module_scope)



from abc import (ABC, abstractmethod)
import os

import rastervision as rv


class VectorSourceDefaultProvider(ABC):
    @staticmethod
    @abstractmethod
    def handles(s):
        """Returns True of this provider is a default for this string"""
        pass

    @abstractmethod
    def construct(s):
        """Constructs a default VectorSource based on the
           string.
        """
        pass


class GeoJSONVectorSourceDefaultProvider(VectorSourceDefaultProvider):
    @staticmethod
    def handles(uri):
        ext = os.path.splitext(uri)[1]
        return ext.lower() in ['.json', '.geojson']

    @staticmethod
    def construct(uri):
        return rv.VectorSourceConfig.builder(rv.GEOJSON_SOURCE) \
                                    .with_uri(uri) \
                                    .build()


class VectorTileVectorSourceDefaultProvider(VectorSourceDefaultProvider):
    @staticmethod
    def handles(uri):
        ext = os.path.splitext(uri)[1]
        return ext.lower() in ['.pbf', '.mvt']

    @staticmethod
    def construct(uri):
        return rv.VectorSourceConfig.builder(rv.VECTOR_TILE_SOURCE) \
                                    .with_uri(uri) \
                                    .build()



import os
import json
import tempfile
from pathlib import Path
import logging

from everett.manager import (ConfigManager, ConfigDictEnv, ConfigEnvFileEnv,
                             ConfigIniEnv, ConfigOSEnv)

import rastervision as rv
from rastervision.cli import Verbosity
from rastervision.filesystem.local_filesystem import make_dir
from rastervision.utils.files import file_to_str

log = logging.getLogger(__name__)


class RVConfig:
    DEFAULT_PROFILE = 'default'

    tmp_dir = None

    @staticmethod
    def get_tmp_dir():
        if RVConfig.tmp_dir is None:
            RVConfig.set_tmp_dir()
        return tempfile.TemporaryDirectory(dir=RVConfig.tmp_dir)

    @staticmethod
    def get_tmp_dir_root():
        if RVConfig.tmp_dir is None:
            RVConfig.set_tmp_dir()
        return RVConfig.tmp_dir

    @staticmethod
    def set_tmp_dir(tmp_dir=None):
        """Set RVConfig.tmp_dir to well-known value.

        This static method sets the value of RVConfig.tmp_dir to some
        well-known value.  The value is chosen from one of the
        following (in order of preference): an explicit value
        (presumably from the command line) is considered first, then
        values from the environment are considered, then the current
        value of RVConfig.tmp_dir is considered, then a directory from
        tempfile.TemporaryDirectory() is considered.

        Args:
            tmp_dir: Either a string or None.

        """
        DEFAULT_DIR = '/opt/data/tmp/'

        # Check the various possibilities in order of priority.
        tmp_dir_array = [tmp_dir]
        env_array = [
            os.environ.get(k) for k in ['TMPDIR', 'TEMP', 'TMP']
            if k in os.environ
        ]
        current_array = [RVConfig.tmp_dir]
        it = tmp_dir_array + env_array + current_array
        it = list(filter(lambda p: p is not None, it))
        if it:
            explicit_tmp_dir = it[0]
        else:
            explicit_tmp_dir = tempfile.TemporaryDirectory().name

        try:
            # Try to create directory
            if not os.path.exists(explicit_tmp_dir):
                os.makedirs(explicit_tmp_dir, exist_ok=True)
            # Check that it is actually a directory
            if not os.path.isdir(explicit_tmp_dir):
                raise Exception(
                    '{} is not a directory.'.format(explicit_tmp_dir))
            # Can we interact with directory?
            Path.touch(Path(os.path.join(explicit_tmp_dir, '.can_touch')))
            # All checks have passed by this point
            RVConfig.tmp_dir = explicit_tmp_dir

        # If directory cannot be made and/or cannot be interacted
        # with, fall back to default.
        except Exception as e:
            log.warning(
                'Root temporary directory cannot be used: {}. Using root: {}'.
                format(explicit_tmp_dir, DEFAULT_DIR))
            RVConfig.tmp_dir = DEFAULT_DIR
        finally:
            make_dir(RVConfig.tmp_dir)
            log.debug('Temporary directory is: {}'.format(RVConfig.tmp_dir))

    @staticmethod
    def get_instance():
        return rv._registry._get_rv_config()

    def __init__(self,
                 profile=None,
                 rv_home=None,
                 config_overrides=None,
                 tmp_dir=None,
                 verbosity=Verbosity.NORMAL):
        self.verbosity = verbosity

        # Set logging level
        root_log = logging.getLogger('rastervision')
        if self.verbosity >= Verbosity.VERBOSE:
            root_log.setLevel(logging.DEBUG)
        elif self.verbosity >= Verbosity.NORMAL:
            root_log.setLevel(logging.INFO)
        else:
            root_log.setLevel(logging.WARN)

        if tmp_dir is not None:
            self.set_tmp_dir(tmp_dir)

        if profile is None:
            if os.environ.get('RV_PROFILE'):
                profile = os.environ.get('RV_PROFILE')
            else:
                profile = RVConfig.DEFAULT_PROFILE

        if config_overrides is None:
            config_overrides = {}

        if rv_home is None:
            home = os.path.expanduser('~')
            rv_home = os.path.join(home, '.rastervision')
        self.rv_home = rv_home

        config_file_locations = self._discover_config_file_locations(profile)

        help_doc = ('Check https://docs.rastervision.io/ for docs.')
        self.config = ConfigManager(
            # Specify one or more configuration environments in
            # the order they should be checked
            [
                # Allow overrides
                ConfigDictEnv(config_overrides),

                # Looks in OS environment first
                ConfigOSEnv(),

                # Look for an .env file
                ConfigEnvFileEnv('.env'),

                # Looks in INI files in order specified
                ConfigIniEnv(config_file_locations),
            ],

            # Make it easy for users to find your configuration docs
            doc=help_doc)

    def _discover_config_file_locations(self, profile):
        result = []

        # Allow for user to specify specific config file
        # in the RASTERVISION_CONFIG env variable.
        env_specified_path = os.environ.get('RV_CONFIG')
        if env_specified_path:
            result.append(env_specified_path)

        # Allow user to specify config directory that will
        # contain profile configs in RASTERVISION_CONFIG_DIR
        # env variable. Otherwise, use "$HOME/.rastervision"
        env_specified_dir_path = os.environ.get('RV_CONFIG_DIR')
        if env_specified_dir_path:
            result.append(os.path.join(env_specified_dir_path, profile))
        else:
            result.append(os.path.join(self.rv_home, profile))
        result.append(os.path.join(os.getcwd(), '.rastervision'))

        # Filter out any that do not exist.
        results_that_exist = list(filter(lambda x: os.path.exists(x), result))

        # If the profile is not default, and there is no config that exists,
        # then throw an error.
        if not any(results_that_exist) and profile != RVConfig.DEFAULT_PROFILE:
            raise rv.ConfigError('Configuration Profile {} not found. '
                                 'Checked: {}'.format(profile,
                                                      ', '.join(result)))

        return results_that_exist

    def get_subconfig(self, namespace):
        return self.config.with_namespace(namespace)

    def get_model_defaults(self):
        """Return the "model defaults"

        The model defaults is a json file that lists a set of default
        configurations for models, per backend and model key.
        There are defaults that are installed with Raster Vision, but
        users can override these defaults with their own by setting
        the "model_defaults_uri" in the [RV] section of
        thier configuration file, or by setting the RV_MODEL_DEFAULT_URI
        environment variable.
        """
        subconfig = self.get_subconfig('RV')
        default_path = os.path.join(
            os.path.dirname(rv.backend.__file__), 'model_defaults.json')
        model_default_uri = subconfig(
            'model_defaults_uri', default=default_path)

        model_defaults = json.loads(file_to_str(model_default_uri))

        return model_defaults

    def get_verbosity(self):
        return self.verbosity



# flake8: noqa

from rastervision.data.activate_mixin import *
from rastervision.data.raster_transformer import *
from rastervision.data.raster_source import *
from rastervision.data.crs_transformer import *
from rastervision.data.label import *
from rastervision.data.vector_source import *
from rastervision.data.label_source import *
from rastervision.data.label_store import *
from rastervision.data.scene import *
from rastervision.data.scene_config import *
from rastervision.data.dataset import *
from rastervision.data.dataset_config import *



import numpy as np
import rasterio

import rastervision as rv
from rastervision.utils.files import (get_local_path, make_dir, upload_or_copy,
                                      file_exists)
from rastervision.data.label import SemanticSegmentationLabels
from rastervision.data.label_store import LabelStore
from rastervision.data.label_source import SegmentationClassTransformer


class SemanticSegmentationRasterStore(LabelStore):
    """A prediction label store for segmentation raster files.
    """

    def __init__(self,
                 uri,
                 extent,
                 crs_transformer,
                 tmp_dir,
                 vector_output=None,
                 class_map=None):
        """Constructor.

        Args:
            uri: (str) URI of GeoTIFF file used for storing predictions as RGB values
            extent: (Box) The extent of the scene
            crs_transformer: (CRSTransformer)
            tmp_dir: (str) temp directory to use
            vector_output: (None or array of dicts) containing vectorifiction
                configuration information
            class_map: (ClassMap) with color values used to convert class ids to
                RGB values

        """
        self.uri = uri
        self.vector_output = vector_output
        self.extent = extent
        self.crs_transformer = crs_transformer
        self.tmp_dir = tmp_dir
        # Note: can't name this class_transformer due to Python using that attribute
        if class_map:
            self.class_trans = SegmentationClassTransformer(class_map)
        else:
            self.class_trans = None

        self.source = None
        if file_exists(uri):
            self.source = rv.RasterSourceConfig.builder(rv.RASTERIO_SOURCE) \
                                               .with_uri(self.uri) \
                                               .build() \
                                               .create_source(self.tmp_dir)

    def _subcomponents_to_activate(self):
        if self.source is not None:
            return [self.source]
        return []

    def get_labels(self, chip_size=1000):
        """Get all labels.

        Returns:
            SemanticSegmentationLabels with windows of size chip_size covering the
                scene with no overlap.
        """

        def label_fn(window):
            raw_labels = self.source.get_raw_chip(window)
            if self.class_trans:
                labels = self.class_trans.rgb_to_class(raw_labels)
            else:
                labels = np.squeeze(raw_labels)
            return labels

        if self.source is None:
            raise Exception('Raster source at {} does not exist'.format(
                self.uri))

        extent = self.source.get_extent()
        windows = extent.get_windows(chip_size, chip_size)
        return SemanticSegmentationLabels(windows, label_fn)

    def save(self, labels):
        """Save.

        Args:
            labels - (SemanticSegmentationLabels) labels to be saved
        """
        local_path = get_local_path(self.uri, self.tmp_dir)
        make_dir(local_path, use_dirname=True)

        transform = self.crs_transformer.get_affine_transform()
        crs = self.crs_transformer.get_image_crs()

        band_count = 1
        dtype = np.uint8
        if self.class_trans:
            band_count = 3

        if self.vector_output:
            # We need to store the whole output mask to run feature extraction.
            # If the raster is large, this will result in running out of memory, so
            # more work will be needed to get this to work in a scalable way. But this
            # is complicated because of the need to merge features that are split
            # across windows.
            mask = np.zeros(
                (self.extent.ymax, self.extent.xmax), dtype=np.uint8)
        else:
            mask = None

        # https://github.com/mapbox/rasterio/blob/master/docs/quickstart.rst
        # https://rasterio.readthedocs.io/en/latest/topics/windowed-rw.html
        with rasterio.open(
                local_path,
                'w',
                driver='GTiff',
                height=self.extent.ymax,
                width=self.extent.xmax,
                count=band_count,
                dtype=dtype,
                transform=transform,
                crs=crs) as dataset:
            for window in labels.get_windows():
                class_labels = labels.get_label_arr(
                    window, clip_extent=self.extent)
                clipped_window = ((window.ymin,
                                   window.ymin + class_labels.shape[0]),
                                  (window.xmin,
                                   window.xmin + class_labels.shape[1]))
                if mask is not None:
                    mask[clipped_window[0][0]:clipped_window[0][1],
                         clipped_window[1][0]:clipped_window[1][
                             1]] = class_labels
                if self.class_trans:
                    rgb_labels = self.class_trans.class_to_rgb(class_labels)
                    for chan in range(3):
                        dataset.write_band(
                            chan + 1,
                            rgb_labels[:, :, chan],
                            window=clipped_window)
                else:
                    img = class_labels.astype(dtype)
                    dataset.write_band(1, img, window=clipped_window)

        upload_or_copy(local_path, self.uri)

        if self.vector_output:
            import mask_to_polygons.vectorification as vectorification
            import mask_to_polygons.processing.denoise as denoise

            for vo in self.vector_output:
                denoise_radius = vo['denoise']
                uri = vo['uri']
                mode = vo['mode']
                class_id = vo['class_id']
                class_mask = np.array(mask == class_id, dtype=np.uint8)
                local_geojson_path = get_local_path(uri, self.tmp_dir)

                def transform(x, y):
                    return self.crs_transformer.pixel_to_map((x, y))

                if denoise_radius > 0:
                    class_mask = denoise.denoise(class_mask, denoise_radius)

                if uri and mode == 'buildings':
                    options = vo['building_options']
                    geojson = vectorification.geojson_from_mask(
                        mask=class_mask,
                        transform=transform,
                        mode=mode,
                        min_aspect_ratio=options['min_aspect_ratio'],
                        min_area=options['min_area'],
                        width_factor=options['element_width_factor'],
                        thickness=options['element_thickness'])
                elif uri and mode == 'polygons':
                    geojson = vectorification.geojson_from_mask(
                        mask=class_mask, transform=transform, mode=mode)

                if local_geojson_path:
                    with open(local_geojson_path, 'w') as file_out:
                        file_out.write(geojson)
                        upload_or_copy(local_geojson_path, uri)

    def empty_labels(self):
        """Returns an empty SemanticSegmentationLabels object."""
        return SemanticSegmentationLabels()



import os
from os.path import join, basename, isfile
import uuid
import zipfile
import glob
import logging
import json
import csv
import time
import datetime
from subprocess import Popen

import matplotlib
matplotlib.use('Agg')  # noqa
import matplotlib.pyplot as plt
import torch
from torch import optim
from torch.utils.tensorboard import SummaryWriter
from torch.optim.lr_scheduler import CyclicLR

from rastervision.utils.files import (get_local_path, make_dir, upload_or_copy,
                                      list_paths, download_if_needed,
                                      sync_from_dir, sync_to_dir, str_to_file,
                                      json_to_file, file_to_json, zipdir)
from rastervision.utils.misc import save_img, terminate_at_exit
from rastervision.backend import Backend
from rastervision.data import ObjectDetectionLabels
from rastervision.backend.torch_utils.object_detection.data import build_databunch
from rastervision.backend.torch_utils.object_detection.plot import plot_xy
from rastervision.backend.torch_utils.object_detection.model import MyFasterRCNN
from rastervision.backend.torch_utils.object_detection.train import (
    train_epoch, validate_epoch)

log = logging.getLogger(__name__)


def make_debug_chips(databunch, class_map, tmp_dir, train_uri, max_count=30):
    """Save debug chips for a DataBunch.

    This saves a plot for each example in the training and validation sets into
    train-debug-chips.zip and valid-debug-chips.zip under the train_uri. This
    is useful for making sure we are feeding correct data into the model.

    Args:
        data: DataBunch for an object detection problem
        class_map: (rv.ClassMap) class map used to map class ids to colors
        tmp_dir: (str) path to temp directory
        train_uri: (str) URI of root of training output
        max_count: (int) maximum number of chips to generate. If None,
            generates all of them.
    """

    def _make_debug_chips(split):
        debug_chips_dir = join(tmp_dir, '{}-debug-chips'.format(split))
        zip_path = join(tmp_dir, '{}-debug-chips.zip'.format(split))
        zip_uri = join(train_uri, '{}-debug-chips.zip'.format(split))
        make_dir(debug_chips_dir)
        ds = databunch.train_ds if split == 'train' else databunch.valid_ds
        for i, (x, y) in enumerate(ds):
            if i >= max_count:
                break

            fig, ax = plt.subplots(1)
            plot_xy(ax, x, y, ds.label_names)
            plt.savefig(
                join(debug_chips_dir, '{}.png'.format(i)), figsize=(6, 6))
            plt.close()

        zipdir(debug_chips_dir, zip_path)
        upload_or_copy(zip_path, zip_uri)

    _make_debug_chips('train')
    _make_debug_chips('val')


class PyTorchObjectDetection(Backend):
    """Object detection using PyTorch and Faster-RCNN/Resnet50 from torchvision."""

    def __init__(self, task_config, backend_opts, train_opts):
        """Constructor.

        Args:
            task_config: (ChipClassificationConfig)
            backend_opts: (simple_backend_config.BackendOptions)
            train_opts: (pytorch_chip_classification_config.TrainOptions)
        """
        self.task_config = task_config
        self.backend_opts = backend_opts
        self.train_opts = train_opts
        self.inf_learner = None

        # Setup caching for torchvision pretrained models.
        torch_cache_dir = '/opt/data/torch-cache'
        os.environ['TORCH_HOME'] = torch_cache_dir

        self.model = None
        self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        log.info('Device = {}'.format(self.device))

    def log_options(self):
        log.info('backend_opts:\n' +
                 json.dumps(self.backend_opts.__dict__, indent=4))
        log.info('train_opts:\n' +
                 json.dumps(self.train_opts.__dict__, indent=4))

    def process_scene_data(self, scene, data, tmp_dir):
        """Process each scene's training data.

        This writes {scene_id}/{scene_id}-{ind}.png and
        {scene_id}/{scene_id}-labels.json in COCO format.

        Args:
            scene: Scene
            data: TrainingData

        Returns:
            backend-specific data-structures consumed by backend's
            process_sceneset_results
        """
        scene_dir = join(tmp_dir, str(scene.id))
        labels_path = join(scene_dir, '{}-labels.json'.format(scene.id))

        make_dir(scene_dir)
        images = []
        annotations = []
        categories = [{
            'id': item.id,
            'name': item.name
        } for item in self.task_config.class_map.get_items()]

        for im_ind, (chip, window, labels) in enumerate(data):
            im_id = '{}-{}'.format(scene.id, im_ind)
            fn = '{}.png'.format(im_id)
            chip_path = join(scene_dir, fn)
            save_img(chip, chip_path)
            images.append({
                'file_name': fn,
                'id': im_id,
                'height': chip.shape[0],
                'width': chip.shape[1]
            })

            npboxes = labels.get_npboxes()
            npboxes = ObjectDetectionLabels.global_to_local(npboxes, window)
            for box_ind, (box, class_id) in enumerate(
                    zip(npboxes, labels.get_class_ids())):
                bbox = [box[1], box[0], box[3] - box[1], box[2] - box[0]]
                bbox = [int(i) for i in bbox]
                annotations.append({
                    'id': '{}-{}'.format(im_id, box_ind),
                    'image_id': im_id,
                    'bbox': bbox,
                    'category_id': int(class_id)
                })

        coco_dict = {
            'images': images,
            'annotations': annotations,
            'categories': categories
        }
        json_to_file(coco_dict, labels_path)

        return scene_dir

    def process_sceneset_results(self, training_results, validation_results,
                                 tmp_dir):
        """After all scenes have been processed, process the result set.

        This writes a zip file for a group of scenes at {chip_uri}/{uuid}.zip
        containing:
        train/{scene_id}-{ind}.png
        train/{scene_id}-labels.json
        valid/{scene_id}-{ind}.png
        valid/{scene_id}-labels.json

        Args:
            training_results: dependent on the ml_backend's process_scene_data
            validation_results: dependent on the ml_backend's
                process_scene_data
        """
        self.log_options()

        group = str(uuid.uuid4())
        group_uri = join(self.backend_opts.chip_uri, '{}.zip'.format(group))
        group_path = get_local_path(group_uri, tmp_dir)
        make_dir(group_path, use_dirname=True)

        with zipfile.ZipFile(group_path, 'w', zipfile.ZIP_DEFLATED) as zipf:

            def _write_zip(results, split):
                for scene_dir in results:
                    scene_paths = glob.glob(join(scene_dir, '*'))
                    for p in scene_paths:
                        zipf.write(p, join(split, basename(p)))

            _write_zip(training_results, 'train')
            _write_zip(validation_results, 'valid')

        upload_or_copy(group_path, group_uri)

    def train(self, tmp_dir):
        """Train a model.

        This downloads any previous output saved to the train_uri,
        starts training (or resumes from a checkpoint), periodically
        syncs contents of train_dir to train_uri and after training finishes.

        Args:
            tmp_dir: (str) path to temp directory
        """
        self.log_options()

        # Sync output of previous training run from cloud.
        train_uri = self.backend_opts.train_uri
        train_dir = get_local_path(train_uri, tmp_dir)
        make_dir(train_dir)
        sync_from_dir(train_uri, train_dir)

        # Get zip file for each group, and unzip them into chip_dir.
        chip_dir = join(tmp_dir, 'chips')
        make_dir(chip_dir)
        for zip_uri in list_paths(self.backend_opts.chip_uri, 'zip'):
            zip_path = download_if_needed(zip_uri, tmp_dir)
            with zipfile.ZipFile(zip_path, 'r') as zipf:
                zipf.extractall(chip_dir)

        # Setup dataset and dataloaders.
        batch_size = self.train_opts.batch_size
        chip_size = self.task_config.chip_size
        databunch = build_databunch(chip_dir, chip_size, batch_size)
        log.info(databunch)
        num_labels = len(databunch.label_names)
        if self.train_opts.debug:
            make_debug_chips(databunch, self.task_config.class_map, tmp_dir,
                             train_uri)

        # Setup model
        num_labels = len(databunch.label_names)
        model = MyFasterRCNN(
            self.train_opts.model_arch, num_labels, chip_size, pretrained=True)
        model = model.to(self.device)
        model_path = join(train_dir, 'model')

        # Load weights from a pretrained model.
        pretrained_uri = self.backend_opts.pretrained_uri
        if pretrained_uri:
            log.info('Loading weights from pretrained_uri: {}'.format(
                pretrained_uri))
            pretrained_path = download_if_needed(pretrained_uri, tmp_dir)
            model.load_state_dict(
                torch.load(pretrained_path, map_location=self.device))

        # Possibly resume training from checkpoint.
        start_epoch = 0
        train_state_path = join(train_dir, 'train_state.json')
        if isfile(train_state_path):
            log.info('Resuming from checkpoint: {}\n'.format(model_path))
            train_state = file_to_json(train_state_path)
            start_epoch = train_state['epoch'] + 1
            model.load_state_dict(
                torch.load(model_path, map_location=self.device))

        # Write header of log CSV file.
        log_path = join(train_dir, 'log.csv')
        if not isfile(log_path):
            with open(log_path, 'w') as log_file:
                log_writer = csv.writer(log_file)
                row = ['epoch'] + ['map50', 'time'] + model.subloss_names
                log_writer.writerow(row)

        # Setup Tensorboard logging.
        if self.train_opts.log_tensorboard:
            log_dir = join(train_dir, 'tb-logs')
            make_dir(log_dir)
            tb_writer = SummaryWriter(log_dir=log_dir)
            if self.train_opts.run_tensorboard:
                log.info('Starting tensorboard process')
                tensorboard_process = Popen(
                    ['tensorboard', '--logdir={}'.format(log_dir)])
                terminate_at_exit(tensorboard_process)

        # Setup optimizer.
        lr = self.train_opts.lr
        opt = optim.Adam(model.parameters(), lr=lr)
        step_scheduler, epoch_scheduler = None, None
        num_epochs = self.train_opts.num_epochs

        if self.train_opts.one_cycle and num_epochs > 1:
            steps_per_epoch = len(databunch.train_ds) // batch_size
            total_steps = num_epochs * steps_per_epoch
            step_size_up = (num_epochs // 2) * steps_per_epoch
            step_size_down = total_steps - step_size_up
            step_scheduler = CyclicLR(
                opt,
                base_lr=lr / 10,
                max_lr=lr,
                step_size_up=step_size_up,
                step_size_down=step_size_down,
                cycle_momentum=False)
            for _ in range(start_epoch * steps_per_epoch):
                step_scheduler.step()

        # Training loop.
        for epoch in range(start_epoch, num_epochs):
            # Train one epoch.
            log.info('-----------------------------------------------------')
            log.info('epoch: {}'.format(epoch))
            start = time.time()
            train_loss = train_epoch(model, self.device, databunch.train_dl,
                                     opt, step_scheduler, epoch_scheduler)
            if epoch_scheduler:
                epoch_scheduler.step()
            log.info('train loss: {}'.format(train_loss))

            # Validate one epoch.
            metrics = validate_epoch(model, self.device, databunch.valid_dl,
                                     num_labels)
            log.info('validation metrics: {}'.format(metrics))

            # Print elapsed time for epoch.
            end = time.time()
            epoch_time = datetime.timedelta(seconds=end - start)
            log.info('epoch elapsed time: {}'.format(epoch_time))

            # Save model and state.
            torch.save(model.state_dict(), model_path)
            train_state = {'epoch': epoch}
            json_to_file(train_state, train_state_path)

            # Append to log CSV file.
            with open(log_path, 'a') as log_file:
                log_writer = csv.writer(log_file)
                row = [epoch]
                row += [metrics['map50'], epoch_time]
                row += [train_loss[k] for k in model.subloss_names]
                log_writer.writerow(row)

            # Write to Tensorboard log.
            if self.train_opts.log_tensorboard:
                for key, val in metrics.items():
                    tb_writer.add_scalar(key, val, epoch)
                for key, val in train_loss.items():
                    tb_writer.add_scalar(key, val, epoch)
                for name, param in model.named_parameters():
                    tb_writer.add_histogram(name, param, epoch)

            if (train_uri.startswith('s3://')
                    and (((epoch + 1) % self.train_opts.sync_interval) == 0)):
                sync_to_dir(train_dir, train_uri)

        # Close Tensorboard.
        if self.train_opts.log_tensorboard:
            tb_writer.close()
            if self.train_opts.run_tensorboard:
                tensorboard_process.terminate()

        # Mark that the command has completed.
        str_to_file('done!', self.backend_opts.train_done_uri)

        # Sync output to cloud.
        sync_to_dir(train_dir, self.backend_opts.train_uri)

    def load_model(self, tmp_dir):
        """Load the model in preparation for one or more prediction calls."""
        if self.model is None:
            model_uri = self.backend_opts.model_uri
            model_path = download_if_needed(model_uri, tmp_dir)

            # add one for background class
            num_classes = len(self.task_config.class_map) + 1
            model = MyFasterRCNN(
                self.train_opts.model_arch,
                num_classes,
                self.task_config.chip_size,
                pretrained=False)
            model = model.to(self.device)
            model.load_state_dict(
                torch.load(model_path, map_location=self.device))
            self.model = model

    def predict(self, chips, windows, tmp_dir):
        """Return predictions for a chip using model.

        Args:
            chips: [[height, width, channels], ...] numpy array of chips
            windows: List of boxes that are the windows aligned with the chips.

        Return:
            Labels object containing predictions
        """
        self.load_model(tmp_dir)
        labels = ObjectDetectionLabels.make_empty()
        chips = torch.Tensor(chips).permute((0, 3, 1, 2)) / 255.
        chips = chips.to(self.device)
        model = self.model.eval()

        with torch.no_grad():
            output = model(chips)
            for chip_ind, (chip, window) in enumerate(zip(chips, windows)):
                boxlist = output[chip_ind].cpu()
                boxes = boxlist.boxes.numpy()
                class_ids = boxlist.get_field('labels').numpy()
                scores = boxlist.get_field('scores').numpy()

                boxes = ObjectDetectionLabels.local_to_global(boxes, window)
                labels += ObjectDetectionLabels(
                    boxes, class_ids, scores=scores)

        return labels



import json
import copy

import click

from rastervision.utils.files import file_to_str, str_to_file


@click.command()
@click.argument('labels_uri')
@click.argument('output_uri')
@click.argument('class_names', nargs=-1)
def filter_geojson(labels_uri, output_uri, class_names):
    """Remove features that aren't in class_names and remove class_ids."""
    labels_str = file_to_str(labels_uri)
    labels = json.loads(labels_str)
    filtered_features = []

    for feature in labels['features']:
        feature = copy.deepcopy(feature)
        properties = feature.get('properties')
        if properties:
            class_name = properties.get('class_name') or properties('label')
            if class_name in class_names:
                del properties['class_id']
                filtered_features.append(feature)

    new_labels = {'features': filtered_features}
    str_to_file(json.dumps(new_labels), output_uri)


if __name__ == '__main__':
    filter_geojson()



import uuid
import click
import logging

from rastervision.runner import OutOfProcessExperimentRunner
from rastervision.rv_config import RVConfig
from rastervision.utils.misc import grouped

DEPENDENCY_GROUP_JOB = 'DEPENDENCY_GROUP'

NOOP_COMMAND = 'python -m rastervision --help'

log = logging.getLogger(__name__)


class AwsBatchExperimentRunner(OutOfProcessExperimentRunner):
    def __init__(self):
        super().__init__()

        rv_config = RVConfig.get_instance()

        batch_config = rv_config.get_subconfig('AWS_BATCH')

        self.attempts = batch_config('attempts', parser=int, default='1')
        self.gpu = batch_config('gpu', parser=bool, default='true')

        job_queue = batch_config('job_queue', default='')
        if not job_queue:
            if self.gpu:
                job_queue = 'raster-vision-gpu'
            else:
                job_queue = 'raster-vision-cpu'
        self.job_queue = job_queue

        cpu_job_queue = batch_config('cpu_job_queue', default='')
        if not cpu_job_queue:
            if self.gpu:
                cpu_job_queue = 'raster-vision-cpu'
            else:
                cpu_job_queue = job_queue
        self.cpu_job_queue = cpu_job_queue

        job_definition = batch_config('job_definition', default='')
        if not job_definition:
            if self.gpu:
                job_definition = 'raster-vision-gpu'
            else:
                job_definition = 'raster-vision-cpu'
        self.job_definition = job_definition

        cpu_job_definition = batch_config('cpu_job_definition', default='')
        if not cpu_job_definition:
            if self.gpu:
                cpu_job_definition = 'raster-vision-cpu'
            else:
                cpu_job_definition = job_definition
        self.cpu_job_definition = cpu_job_definition

        self.submit = self.batch_submit
        self.execution_environment = 'Batch'

    def _get_boto_client(self):
        """Returns a boto client.
        Encapsulated in a method for testability.
        """
        import boto3
        return boto3.client('batch')

    def batch_submit(self,
                     command_type,
                     command_split_id,
                     experiment_id,
                     command,
                     parent_job_ids=None,
                     array_size=None,
                     utilizes_gpu=False):
        """
        Submit a job to run on Batch.

        Args:
           command_type: (str) the type of command. ie. a value in rv.command.api
           command_split_id: (int or str) the split ID of command.
                             ie. the parallelized command ID
           experiment_id: (str) id of experiment
           command: (str) command to run inside Docker container
           parent_job_ids (list of str): ids of jobs that this job depends on
           array_size: (int) size of the Batch array job
        """
        if parent_job_ids is None:
            parent_job_ids = []

        if command_type == DEPENDENCY_GROUP_JOB:
            full_command = NOOP_COMMAND.split()
        else:
            full_command = command.split()

        # When running on Batch, we set the temp dir to be inside /opt/data
        # since it is mounted to the host instance which has much more disk
        # space.
        full_command += ['--tempdir', '/opt/data/temp/']

        client = self._get_boto_client()

        uuid_part = str(uuid.uuid4()).split('-')[0]
        if experiment_id is None:
            exp = 'COMMAND'
        else:
            exp = ''.join(e for e in experiment_id if e.isalnum())
        job_name = '{}-{}_{}_{}'.format(command_type, command_split_id, exp,
                                        uuid_part)

        group_level = 1
        while len(parent_job_ids) > 20:
            # AWS Batch only allows for 20 parent jobs.
            # This hacks around that limit.
            # Group dependencies  into batches of 20,
            # and submit noop jobs that form a graph
            # of dependencies where no set of leaves is
            # greater than 20.
            log.warn('More that 20 parent jobs detected, grouping [LEVEL {}]'.
                     format(group_level))
            new_parents = []
            group_id = str(uuid.uuid4()).split('-')[0]
            for i, group in enumerate(grouped(parent_job_ids, 20)):
                group_split_id = '{}_{}-{}_{}'.format(group_id, command_type,
                                                      command_split_id, i)
                new_parents.append(
                    self.batch_submit(DEPENDENCY_GROUP_JOB, group_split_id,
                                      experiment_id, NOOP_COMMAND, group,
                                      array_size))
            parent_job_ids = new_parents
            group_level += 1

        depends_on = [{'jobId': job_id} for job_id in parent_job_ids]

        if utilizes_gpu:
            job_queue = self.job_queue
            job_definition = self.job_definition
        else:
            job_queue = self.cpu_job_queue
            job_definition = self.cpu_job_definition

        kwargs = {
            'jobName': job_name,
            'jobQueue': job_queue,
            'jobDefinition': job_definition,
            'containerOverrides': {
                'command': full_command
            },
            'retryStrategy': {
                'attempts': self.attempts
            },
            'dependsOn': depends_on
        }

        if array_size is not None:
            kwargs['arrayProperties'] = {'size': array_size}

        job_id = client.submit_job(**kwargs)['jobId']

        msg = '{}-{} command submitted job with jobName={} and jobId={}'.format(
            command_type, command_split_id, job_name, job_id)
        click.echo(click.style(msg, fg='green'))

        return job_id



import os
from copy import deepcopy
import json

from google.protobuf import struct_pb2

import rastervision as rv
from rastervision.rv_config import RVConfig
from rastervision.command import (CommandConfig, CommandConfigBuilder)
from rastervision.protos.command_pb2 \
    import CommandConfig as CommandConfigMsg
from rastervision.utils.misc import split_into_groups


class AuxCommandConfig(CommandConfig):
    def __init__(self, command_class, root_uri, config, split_id=0):
        super().__init__(command_class.command_type, root_uri)
        self.command_class = command_class
        self.command_options = command_class.options
        self.config = config
        self.split_id = split_id

    def create_command(self, tmp_dir=None):
        if not tmp_dir:
            _tmp_dir = RVConfig.get_tmp_dir()
            tmp_dir = _tmp_dir.name
        else:
            _tmp_dir = tmp_dir

        retval = self.command_class(self.config)
        retval.set_tmp_dir(_tmp_dir)

        return retval

    def to_proto(self):
        msg = super().to_proto()
        conf = struct_pb2.Struct()
        conf['json'] = json.dumps(self.config)
        msg.MergeFrom(CommandConfigMsg(custom_config=conf))

        return msg

    def report_io(self):
        io_def = rv.core.CommandIODefinition()
        inputs = self.command_options.inputs(self.config)
        outputs = self.command_options.outputs(self.config)

        if inputs:
            io_def.add_inputs(inputs)
        if outputs:
            io_def.add_outputs(outputs)

        return io_def

    def split(self, num_parts):
        split_on = self.command_options.split_on
        if split_on:
            commands = []
            for i, split_elements in enumerate(
                    split_into_groups(self.config[split_on], num_parts)):
                split_config = deepcopy(self.config)
                split_config[split_on] = split_elements
                c = self.to_builder() \
                        .with_config(**split_config) \
                        .with_split_id(i) \
                        .build()
                commands.append(c)
            return commands
        else:
            return [self]


class AuxCommandConfigBuilder(CommandConfigBuilder):
    def __init__(self, command_type, prev=None):
        super().__init__(command_type, prev)
        if prev:
            self.config = prev.config
            self.command_class = prev.command_class
        else:
            self.config = None
            self.command_class = None

    def validate(self):
        super().validate()

        if not self.command_class:
            raise rv.ConfigError(
                'AuxCommandConfigBuilder requires the command_class be set.')

        if self.config is None:
            raise rv.ConfigError(
                'AuxCommandConfigBuilder requires a configuration be set, either '
                'through with_config or by setting a dict with the property "config" '
                'in an experiment custom configuration dict with the command name '
                'as the key in the experiment custom configuration')

        if self.command_class.options.required_fields:
            for field in self.command_class.options.required_fields:
                if field not in self.config:
                    raise rv.ConfigError('{} command requires the field {} '
                                         'be set in the configuration.'.format(
                                             self.command_type, field))

    def build(self):
        self.validate()
        return AuxCommandConfig(self.command_class, self.root_uri, self.config,
                                self.split_id)

    def from_proto(self, msg):
        b = super().from_proto(msg)

        self.command_class = rv._registry.get_aux_command_class(
            self.command_type)

        b = b.with_config(**json.loads(msg.custom_config['json']))

        return b

    def _get_config_from_experiment(self, experiment_config):
        command_name = self.command_type.lower()
        matching_props = list(
            filter(lambda x: x.lower() == command_name,
                   experiment_config.custom_config.keys()))
        if len(matching_props) > 1:
            raise rv.ConfigError(
                'Multiple configurations matching the {} command found. '
                'Use only one configuration - property match is not case sensitive.'.
                format(self.command_type))
        elif len(matching_props) == 1:
            return experiment_config.custom_config[matching_props[0]]
        else:
            return None

    def get_root_uri(self, experiment_config):
        if self.root_uri:
            return self.root_uri

        command_name = self.command_type.lower()
        command_config = self._get_config_from_experiment(experiment_config)

        if not command_config:
            raise rv.ConfigError(
                '{} command requires experiment custom_config '
                'contains a {} key'.format(self.command_type, command_name))
        key = command_config.get('key')
        if not key:
            root_uri = command_config.get('root_uri')
            if not root_uri:
                raise rv.ConfigError(
                    '{} command requires a "key" or "root_uri" '
                    'be set in the command config dict inside the '
                    'experiment custom_config'.format(self.command_type))
        else:
            root_uri = os.path.join(experiment_config.root_uri, command_name,
                                    key)

        return root_uri

    def with_experiment(self, experiment_config):
        b = super().with_experiment(experiment_config)

        command_config = self._get_config_from_experiment(experiment_config)

        if command_config:
            config_data = command_config.get('config')
            if not config_data:
                raise rv.ConfigError(
                    '{} command requires a configuration dict set in the '
                    '"config" property of the experiment '
                    'custom_config for this command.'.format(
                        self.command_type))
            else:
                b = b.with_config(**config_data)

        return b

    def with_config(self, **kwargs):
        b = deepcopy(self)
        b.config = kwargs
        return b

    def with_command_class(self, command_class):
        b = deepcopy(self)
        b.command_class = command_class
        return b



# flake8: noqa

from rastervision.data.raster_transformer.raster_transformer import *
from rastervision.data.raster_transformer.raster_transformer_config import *
from rastervision.data.raster_transformer.noop_transformer import *
from rastervision.data.raster_transformer.stats_transformer import *
from rastervision.data.raster_transformer.stats_transformer_config import *



import os
from copy import deepcopy

import rastervision as rv
from rastervision.analyzer import (AnalyzerConfig, AnalyzerConfigBuilder,
                                   StatsAnalyzer)
from rastervision.protos.analyzer_pb2 import AnalyzerConfig as AnalyzerConfigMsg


class StatsAnalyzerConfig(AnalyzerConfig):
    def __init__(self, stats_uri=None, sample_prob=None):
        super().__init__(rv.STATS_ANALYZER)
        self.stats_uri = stats_uri
        self.sample_prob = sample_prob

    def create_analyzer(self):
        if not self.stats_uri:
            raise rv.ConfigError('stats_uri is not set.')
        return StatsAnalyzer(self.stats_uri, self.sample_prob)

    def to_proto(self):
        msg = AnalyzerConfigMsg(analyzer_type=self.analyzer_type)
        if self.stats_uri:
            msg.stats_analyzer_config.stats_uri = self.stats_uri
        msg.stats_analyzer_config.sample_prob = \
            (0.0 if self.sample_prob is None else self.sample_prob)
        return msg

    def save_bundle_files(self, bundle_dir):
        if not self.stats_uri:
            raise rv.ConfigError('stat_uri is not set.')
        # Only set the basename, do not contribute file
        # as it is not and input and only an output of
        # this analyzer. The StatsTransformer will save
        # its input separately.
        base_name = os.path.basename(self.stats_uri)
        new_config = self.to_builder() \
                         .with_stats_uri(base_name) \
                         .build()
        return (new_config, [])

    def load_bundle_files(self, bundle_dir):
        if not self.stats_uri:
            raise rv.ConfigError('stat_uri is not set.')
        local_stats_uri = os.path.join(bundle_dir, self.stats_uri)
        return self.to_builder() \
                   .with_stats_uri(local_stats_uri) \
                   .build()

    def update_for_command(self, command_type, experiment_config,
                           context=None):
        if command_type == rv.ANALYZE:
            if not self.stats_uri:
                self.stats_uri = os.path.join(experiment_config.analyze_uri,
                                              'stats.json')

    def report_io(self, command_type, io_def):
        if command_type == rv.ANALYZE:
            io_def.add_output(self.stats_uri)


class StatsAnalyzerConfigBuilder(AnalyzerConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'stats_uri': prev.stats_uri,
                'sample_prob': prev.sample_prob
            }
        super().__init__(StatsAnalyzerConfig, config)

    def from_proto(self, msg):
        stats_uri = (msg.stats_analyzer_config.stats_uri or msg.stats_uri)
        b = self.with_stats_uri(stats_uri)

        sample_prob = msg.stats_analyzer_config.sample_prob
        sample_prob = (None if sample_prob == 0 else sample_prob)
        b = b.with_sample_prob(sample_prob)
        return b

    def with_stats_uri(self, stats_uri):
        """Set the stats_uri.

            Args:
                stats_uri: URI to the stats json to use
        """
        b = deepcopy(self)
        b.config['stats_uri'] = stats_uri
        return b

    def validate(self):
        sample_prob = self.config.get('sample_prob')
        if sample_prob and (not isinstance(sample_prob, float)
                            or sample_prob >= 1.0 or sample_prob <= 0):
            raise rv.ConfigError(
                'sample_prob must be a float between 0 and 1 exclusive.')

    def with_sample_prob(self, sample_prob):
        """Set the sample_prob used to sample a subset of each scene.

        If sample_prob is set, then a subset of each scene is used to compute stats which
        speeds up the computation. Roughly speaking, if sample_prob=0.5, then half the
        pixels in the scene will be used. More precisely, the number of chips is equal to
        sample_prob * (width * height / 300^2), or 1, whichever is greater. Each chip is
        uniformly sampled from the scene with replacement. Otherwise, it uses a sliding
        window over the entire scene to compute stats.

        Args:
            sample_prob: (float or None) between 0 and 1
        """
        b = deepcopy(self)
        b.config['sample_prob'] = sample_prob
        return b



import os
from copy import deepcopy

import rastervision as rv
from rastervision.data.label_store import (
    LabelStoreConfig, LabelStoreConfigBuilder, SemanticSegmentationRasterStore)
from rastervision.protos.label_store_pb2 import LabelStoreConfig as LabelStoreConfigMsg

VectorOutput = LabelStoreConfigMsg.SemanticSegmentationRasterStore.VectorOutput


class SemanticSegmentationRasterStoreConfig(LabelStoreConfig):
    def __init__(self, uri=None, vector_output=[], rgb=False):
        super().__init__(store_type=rv.SEMANTIC_SEGMENTATION_RASTER)
        self.uri = uri
        self.vector_output = vector_output
        self.rgb = rgb

    def to_proto(self):
        """Turn this configuration into a ProtoBuf message.

        The fields in the message are as follows:
            - `denoise` gives the radius of the structural element
              used to remove high-frequency signals from the image.
            - `uri` is the location where vector output should be
              written
            - `mode` is the vectorification mode (currently only
              "polygons" and "buildings" are acceptable values).
            - `class_id` specifies the predication class that is to
              turned into vectors
            - `building_options` communicates options useful for
              vectorification of building predictions (it is intended
              to break-up clusters of buildings):
                - `min_aspect_ratio` is the ratio between length and
                  height (or height and length) of anything that can
                  be considered to be a cluster of buildings.  The
                  goal is to distinguish between rows of buildings and
                  (say) a single building.
                - `min_area` is the minimum area of anything that can
                  be considered to be a cluster of buildings.  The
                  goal is to distinguish between buildings and
                  artifacts.
                - `element_width_factor` is the width of the
                  structural element used to break building clusters
                  as a fraction of the width of the cluster.
                - `element_thickness` is the thickness of the
                  structural element that is used to break building
                  clusters.
        """
        msg = super().to_proto()
        if self.uri:
            msg.semantic_segmentation_raster_store.uri = self.uri
        if self.vector_output:
            ar = []
            for vo in self.vector_output:
                vo_msg = VectorOutput()
                vo_msg.denoise = vo['denoise'] if 'denoise' in vo.keys() else 0
                vo_msg.uri = vo['uri'] if 'uri' in vo.keys() else ''
                vo_msg.mode = vo['mode']
                vo_msg.class_id = vo['class_id']
                if 'building_options' in vo.keys():
                    options = vo['building_options']
                else:
                    options = {}
                bldg_msg = vo_msg.building_options
                if 'min_aspect_ratio' in options.keys():
                    bldg_msg.min_aspect_ratio = options['min_aspect_ratio']
                if 'min_area' in options.keys() and options['min_area']:
                    bldg_msg.min_area = options['min_area']
                if 'element_width_factor' in options.keys():
                    bldg_msg.element_width_factor = options[
                        'element_width_factor']
                if 'element_thickness' in options.keys():
                    bldg_msg.element_thickness = options['element_thickness']
                ar.append(vo_msg)
            msg.semantic_segmentation_raster_store.vector_output.extend(ar)
        msg.semantic_segmentation_raster_store.rgb = self.rgb
        return msg

    def for_prediction(self, label_uri):
        return self.to_builder() \
                   .with_uri(label_uri) \
                   .build()

    def create_store(self, task_config, extent, crs_transformer, tmp_dir):
        class_map = None
        if self.rgb:
            class_map = task_config.class_map

        return SemanticSegmentationRasterStore(
            self.uri,
            extent,
            crs_transformer,
            tmp_dir,
            vector_output=self.vector_output,
            class_map=class_map)

    def update_for_command(self, command_type, experiment_config,
                           context=None):
        if command_type == rv.PREDICT:
            if not self.uri:
                # Construct the URI for this prediction store,
                # using the scene ID.
                root = experiment_config.predict_uri
                uri = None
                for c in context:
                    if isinstance(c, rv.SceneConfig):
                        uri = os.path.join(root, '{}.tif'.format(c.id))
                if uri:
                    self.uri = uri
                else:
                    raise rv.ConfigError(
                        'SemanticSegmentationRasterStoreConfig has no '
                        'URI set, and is not associated with a SceneConfig.')

            # Construct URIs for vector predictions
            for vo in self.vector_output:
                for c in context:
                    if isinstance(c,
                                  rv.SceneConfig) and ('uri' not in vo.keys()
                                                       or not vo['uri']):
                        root = experiment_config.predict_uri
                        mode = vo['mode']
                        class_id = vo['class_id']
                        vo['uri'] = os.path.join(
                            root, '{}-{}-{}.geojson'.format(
                                c.id, class_id, mode))

    def report_io(self, command_type, io_def):
        if command_type == rv.PREDICT:
            # Construct URIs for vector predictions
            for vo in self.vector_output:
                io_def.add_output(vo['uri'])
            io_def.add_output(self.uri)

        if command_type == rv.EVAL:
            if self.uri:
                io_def.add_input(self.uri)
            else:
                msg = 'No URI set for SemanticSegmentationRasterStoreConfig'
                io_def.add_missing(msg)

            for vo in self.vector_output:
                io_def.add_input(vo['uri'])


class SemanticSegmentationRasterStoreConfigBuilder(LabelStoreConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'uri': prev.uri,
                'vector_output': prev.vector_output,
                'rgb': prev.rgb,
            }

        super().__init__(SemanticSegmentationRasterStoreConfig, config)
        self.valid_modes = set(['buildings', 'polygons'])

    def from_proto(self, msg):
        uri = msg.semantic_segmentation_raster_store.uri
        rgb = msg.semantic_segmentation_raster_store.rgb
        vo_msg = msg.semantic_segmentation_raster_store.vector_output

        return self.with_uri(uri) \
                   .with_vector_output(vo_msg) \
                   .with_rgb(rgb)

    def with_uri(self, uri):
        """Set URI for a GeoTIFF used to read/write predictions."""
        b = deepcopy(self)
        b.config['uri'] = uri
        return b

    def with_vector_output(self, vector_output):
        """Configure vector output for predictions.

            Args:
                vector_output: Either a list of dictionaries or a
                    protobuf object.  The dictionary or the object
                    contain (respectively) keys (attributes) called
                    'denoise', 'uri', 'class_id', and 'mode'.  The
                    value associated with the 'denoise' key specifies
                    the radius of the structural element used to
                    perform a low-pass filtering process on the mask
                    (see
                    https://en.wikipedia.org/wiki/Mathematical_morphology#Opening).
                    The value associated with the 'uri' key is either
                    a file where the GeoJSON prediction will be
                    written, or "" indicating that the filename should
                    be auto-generated.  'class_id' is the integer
                    prediction class that is of interest.  The 'mode'
                    key must be set to 'buildings' or 'polygons'.

        """
        b = deepcopy(self)
        ar = []

        if isinstance(vector_output, list):
            for vo in vector_output:
                ar.append(vo.copy())
        else:
            for vo_msg in vector_output:
                bldg_msg = vo_msg.building_options
                ar.append({
                    'denoise': vo_msg.denoise,
                    'uri': vo_msg.uri,
                    'mode': vo_msg.mode,
                    'class_id': vo_msg.class_id,
                    'building_options': {
                        'min_aspect_ratio':
                        bldg_msg.min_aspect_ratio,
                        'min_area':
                        bldg_msg.min_area if bldg_msg.min_area > 0 else None,
                        'element_width_factor':
                        bldg_msg.element_width_factor,
                        'element_thickness':
                        bldg_msg.element_thickness,
                    },
                })

        b.config['vector_output'] = ar
        return b

    def with_rgb(self, rgb):
        """Set flag for writing RGB data using the class map.

        Otherwise this method will write the class ID into a single band.
        """
        b = deepcopy(self)
        b.config['rgb'] = rgb
        return b

    def validate(self):
        vector_output = self.config.get('vector_output')

        if vector_output and not isinstance(vector_output, list):
            for vo in vector_output:
                if not hasattr(vo, 'mode'):
                    raise rv.ConfigError(
                        'The attribute vector_output of'
                        ' SemanticSegmentationRasterStoreConfig'
                        ' must be either trivial, a protobuf configuration'
                        ' object, or a list of'
                        ' appropriate dictionaries.')
                if vo.mode not in self.valid_modes:
                    raise rv.ConfigError(
                        'mode key in vector_output dictionary must be one of {}'
                        .format(self.valid_modes))
        elif vector_output and isinstance(vector_output, list):
            for vo in vector_output:
                if not isinstance(vo, dict):
                    raise rv.ConfigError(
                        'The attribute vector_output of'
                        ' SemanticSegmentationRasterStoreConfig'
                        ' must be either trivial, a protobuf configuration'
                        ' object, or a list of'
                        ' appropriate dictionaries.')
                if 'mode' not in vo.keys(
                ) or vo['mode'] not in self.valid_modes:
                    raise rv.ConfigError(
                        'mode key in vector_output dictionary must be one of {}'
                        .format(self.valid_modes))



from subprocess import Popen
import sys
import logging
import os

from rastervision.runner import OutOfProcessExperimentRunner
from rastervision.runner import make_command
from rastervision.utils.misc import (terminate_at_exit)
from rastervision.utils.files import (save_json_config, make_dir)
from rastervision.rv_config import RVConfig

log = logging.getLogger(__name__)


class LocalExperimentRunner(OutOfProcessExperimentRunner):
    def __init__(self, tmp_dir=None):
        super().__init__()

        self.submit = None
        self.execution_environment = 'Shell'
        self.tmp_dir = tmp_dir

    def _run_experiment(self, command_dag):
        tmp_dir = self.tmp_dir or RVConfig.get_tmp_dir().name
        make_dir(tmp_dir)
        makefile_name = os.path.join(tmp_dir, 'Makefile')
        with open(makefile_name, 'w') as makefile:
            command_ids = command_dag.get_sorted_command_ids()

            # .PHONY: 0 1 2 3 4 5
            makefile.write('.PHONY:')
            for command_id in command_ids:
                makefile.write(' {}'.format(command_id))
            makefile.write('\n\n')

            # all: 0 1 2 3 4 5
            makefile.write('all:')
            for command_id in command_ids:
                makefile.write(' {}'.format(command_id))
            makefile.write('\n\n')

            for command_id in command_ids:
                # 0: 1 2
                makefile.write('{}:'.format(command_id))
                for upstream_id in command_dag.get_upstream_command_ids(
                        command_id):
                    makefile.write(' {}'.format(upstream_id))
                makefile.write('\n')

                # \t rastervision ...
                command_def = command_dag.get_command_definition(command_id)
                command_config = command_def.command_config
                command_root_uri = command_config.root_uri
                command_basename = 'command-config-{}.json'.format(
                    command_config.split_id)
                command_uri = os.path.join(command_root_uri, command_basename)
                print('Saving command configuration to {}...'.format(
                    command_uri))
                save_json_config(command_config.to_proto(), command_uri)
                run_command = make_command(command_uri, self.tmp_dir)
                makefile.write('\t{}\n\n'.format(run_command))

        process = Popen(['make', '-j', '-f', makefile_name])
        terminate_at_exit(process)
        exitcode = process.wait()
        if exitcode != 0:
            sys.exit(exitcode)
        else:
            return 0



from copy import deepcopy

import rastervision as rv
from rastervision.data.raster_source.rasterio_source import RasterioSource
from rastervision.data.raster_source.raster_source_config \
    import (RasterSourceConfig, RasterSourceConfigBuilder)
from rastervision.protos.raster_source_pb2 \
    import RasterSourceConfig as RasterSourceConfigMsg
from rastervision.utils.files import download_if_needed


class RasterioSourceConfig(RasterSourceConfig):
    def __init__(self,
                 uris,
                 x_shift_meters=0.0,
                 y_shift_meters=0.0,
                 transformers=None,
                 channel_order=None):
        super().__init__(
            source_type=rv.RASTERIO_SOURCE,
            transformers=transformers,
            channel_order=channel_order)
        self.uris = uris
        self.x_shift_meters = x_shift_meters
        self.y_shift_meters = y_shift_meters

    def to_proto(self):
        msg = super().to_proto()
        msg.rasterio_source.CopyFrom(
            RasterSourceConfigMsg.RasterioSource(
                uris=self.uris,
                x_shift_meters=self.x_shift_meters,
                y_shift_meters=self.y_shift_meters))
        return msg

    def save_bundle_files(self, bundle_dir):
        (conf, files) = super().save_bundle_files(bundle_dir)

        # Replace the URI with a template value.
        new_config = conf.to_builder() \
                         .with_uri('BUNDLE') \
                         .build()
        return (new_config, files)

    def for_prediction(self, image_uri):
        return self.to_builder() \
                   .with_uri(image_uri) \
                   .build()

    def create_local(self, tmp_dir):
        new_uris = [download_if_needed(uri, tmp_dir) for uri in self.uris]
        return self.to_builder() \
                   .with_uris(new_uris) \
                   .build()

    def create_source(self,
                      tmp_dir,
                      crs_transformer=None,
                      extent=None,
                      class_map=None):
        transformers = self.create_transformers()
        x_shift_meters = self.x_shift_meters
        y_shift_meters = self.y_shift_meters
        return RasterioSource(
            uris=self.uris,
            raster_transformers=transformers,
            temp_dir=tmp_dir,
            channel_order=self.channel_order,
            x_shift_meters=x_shift_meters,
            y_shift_meters=y_shift_meters)

    def report_io(self, command_type, io_def):
        super().report_io(command_type, io_def)
        io_def.add_inputs(self.uris)


class RasterioSourceConfigBuilder(RasterSourceConfigBuilder):
    """This RasterSource can read any file that can be opened by Rasterio/GDAL.

    This includes georeferenced formats such as GeoTIFF and non-georeferenced formats
    such as JPG. See https://www.gdal.org/formats_list.html for more details.
    """

    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'uris': prev.uris,
                'transformers': prev.transformers,
                'channel_order': prev.channel_order,
                'x_shift_meters': prev.x_shift_meters,
                'y_shift_meters': prev.y_shift_meters,
            }

        super().__init__(RasterioSourceConfig, config)

    def validate(self):
        super().validate()
        if self.config.get('uris') is None:
            raise rv.ConfigError(
                'You must specify uris for the RasterioSourceConfig. Use '
                '"with_uris".')
        if not isinstance(self.config.get('uris'), list):
            raise rv.ConfigError(
                'uris set with "with_uris" must be a list, got {}'.format(
                    type(self.config.get('uris'))))
        for uri in self.config.get('uris'):
            if not isinstance(uri, str):
                raise rv.ConfigError('uri must be a string, got {}'.format(
                    type(uri)))

    def from_proto(self, msg):
        b = super().from_proto(msg)

        # Need to do this for backward compatibility.
        if msg.HasField('geotiff_files') or msg.HasField('rasterio_source'):
            if msg.HasField('geotiff_files'):
                source = msg.geotiff_files
            else:
                source = msg.rasterio_source

            return b \
                .with_uris(source.uris) \
                .with_shifts(source.x_shift_meters, source.y_shift_meters)
        elif msg.HasField('image_file'):
            source = msg.image_file
            return b.with_uri(source.uri)
        else:
            raise rv.ConfigError(
                'RasterioSourceConfig protobuf message should contain geotiff_files, '
                'rasterio_source, or image_file')

    def with_uris(self, uris):
        """Set URIs for raster files that can be read by Rasterio."""
        b = deepcopy(self)
        b.config['uris'] = list(uris)
        return b

    def with_uri(self, uri):
        """Set URI for raster files that can be read by Rasterio."""
        b = deepcopy(self)
        b.config['uris'] = [uri]
        return b

    def with_shifts(self, x, y):
        """Set the x- and y-shifts in meters.

        This will only have an effect on georeferenced imagery.

        Args:
            x: A number of meters to shift along the x-axis.  A
                positive shift moves the "camera" to the right.

            y: A number of meters to shift along the y-axis.  A
                positive shift moves the "camera" down.
        """
        b = deepcopy(self)
        b.config['x_shift_meters'] = x
        b.config['y_shift_meters'] = y
        return b



from copy import deepcopy

import rastervision as rv
from rastervision.data.vector_source.vector_source_config import (
    VectorSourceConfig, VectorSourceConfigBuilder)
from rastervision.data.vector_source.vector_tile_vector_source import (
    VectorTileVectorSource)
from rastervision.data.vector_source.class_inference import ClassInferenceOptions


class VectorTileVectorSourceConfig(VectorSourceConfig):
    def __init__(self,
                 uri,
                 zoom,
                 id_field,
                 class_id_to_filter=None,
                 default_class_id=1,
                 line_bufs=None,
                 point_bufs=None):
        self.uri = uri
        self.zoom = zoom
        self.id_field = id_field
        super().__init__(
            rv.VECTOR_TILE_SOURCE,
            class_id_to_filter=class_id_to_filter,
            default_class_id=default_class_id,
            line_bufs=line_bufs,
            point_bufs=point_bufs)

    def to_proto(self):
        msg = super().to_proto()
        msg.mbtiles.uri = self.uri
        msg.mbtiles.zoom = self.zoom
        msg.mbtiles.id_field = self.id_field
        return msg

    def create_source(self, crs_transformer=None, extent=None, class_map=None):
        return VectorTileVectorSource(
            self.uri,
            self.zoom,
            self.id_field,
            crs_transformer,
            extent,
            line_bufs=self.line_bufs,
            point_bufs=self.point_bufs,
            class_inf_opts=ClassInferenceOptions(
                class_map=class_map,
                class_id_to_filter=self.class_id_to_filter,
                default_class_id=self.default_class_id))

    def report_io(self, command_type, io_def):
        # We shouldn't include self.uri as an input because it is just a URI schema
        # and the file checker will raise an error if it's included.
        pass


class VectorTileVectorSourceConfigBuilder(VectorSourceConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'uri': prev.uri,
                'zoom': prev.zoom,
                'id_field': prev.id_field,
                'class_id_to_filter': prev.class_id_to_filter,
                'default_class_id': prev.default_class_id,
                'line_bufs': prev.line_bufs,
                'point_bufs': prev.point_bufs
            }

        super().__init__(VectorTileVectorSourceConfig, config)

    def validate(self):
        if self.config.get('uri') is None:
            raise rv.ConfigError(
                'VectorTileVectorSourceConfigBuilder requires uri which '
                'can be set using "with_uri".')

        if self.config.get('zoom') is None:
            raise rv.ConfigError(
                'VectorTileVectorSourceConfigBuilder requires zoom which '
                'can be set using "with_zoom".')

        # If not set explicitly, set it using default value.
        if self.config.get('id_field') is None:
            self.with_id_field()

        super().validate()

    def from_proto(self, msg):
        b = super().from_proto(msg)
        b = b.with_uri(msg.mbtiles.uri)
        b = b.with_zoom(msg.mbtiles.zoom)
        b = b.with_id_field(msg.mbtiles.id_field)
        return b

    def with_uri(self, uri):
        """Set the URI of the vector tiles.

        Args:
            uri: (str) URI of vector tile endpoint. Should either contain
                {z}/{x}/{y} or point to .mbtiles file.
        """
        b = deepcopy(self)
        b.config['uri'] = uri
        return b

    def with_zoom(self, zoom):
        """Set the zoom level to use when accessing vector tiles.

        Note: the vector tiles need to support the zoom level. Typically only
        a subset of zoom levels are supported.
        """
        b = deepcopy(self)
        b.config['zoom'] = zoom
        return b

    def with_id_field(self, id_field='@id'):
        """Set the name of the id field.

        Args:
            id_field: (str) name of field in feature['properties'] that
                contains the feature's unique id. Used for merging features
                that are split across tile boundaries.
        """
        b = deepcopy(self)
        b.config['id_field'] = id_field
        return b



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/post_processing.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/post_processing.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n=rastervision/protos/tf_object_detection/post_processing.proto\x12\'rastervision.protos.tf_object_detection\"\xbc\x01\n\x16\x42\x61tchNonMaxSuppression\x12\x1a\n\x0fscore_threshold\x18\x01 \x01(\x02:\x01\x30\x12\x1a\n\riou_threshold\x18\x02 \x01(\x02:\x03\x30.6\x12%\n\x18max_detections_per_class\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12!\n\x14max_total_detections\x18\x05 \x01(\x05:\x03\x31\x30\x30\x12 \n\x11use_static_shapes\x18\x06 \x01(\x08:\x05\x66\x61lse\"\xb1\x02\n\x0ePostProcessing\x12\x62\n\x19\x62\x61tch_non_max_suppression\x18\x01 \x01(\x0b\x32?.rastervision.protos.tf_object_detection.BatchNonMaxSuppression\x12i\n\x0fscore_converter\x18\x02 \x01(\x0e\x32\x46.rastervision.protos.tf_object_detection.PostProcessing.ScoreConverter:\x08IDENTITY\x12\x16\n\x0blogit_scale\x18\x03 \x01(\x02:\x01\x31\"8\n\x0eScoreConverter\x12\x0c\n\x08IDENTITY\x10\x00\x12\x0b\n\x07SIGMOID\x10\x01\x12\x0b\n\x07SOFTMAX\x10\x02')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_POSTPROCESSING_SCORECONVERTER = _descriptor.EnumDescriptor(
  name='ScoreConverter',
  full_name='rastervision.protos.tf_object_detection.PostProcessing.ScoreConverter',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IDENTITY', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SIGMOID', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SOFTMAX', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=547,
  serialized_end=603,
)
_sym_db.RegisterEnumDescriptor(_POSTPROCESSING_SCORECONVERTER)


_BATCHNONMAXSUPPRESSION = _descriptor.Descriptor(
  name='BatchNonMaxSuppression',
  full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='score_threshold', full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression.score_threshold', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='iou_threshold', full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression.iou_threshold', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.6),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_detections_per_class', full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression.max_detections_per_class', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=100,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_total_detections', full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression.max_total_detections', index=3,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=100,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_static_shapes', full_name='rastervision.protos.tf_object_detection.BatchNonMaxSuppression.use_static_shapes', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=295,
)


_POSTPROCESSING = _descriptor.Descriptor(
  name='PostProcessing',
  full_name='rastervision.protos.tf_object_detection.PostProcessing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_non_max_suppression', full_name='rastervision.protos.tf_object_detection.PostProcessing.batch_non_max_suppression', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score_converter', full_name='rastervision.protos.tf_object_detection.PostProcessing.score_converter', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='logit_scale', full_name='rastervision.protos.tf_object_detection.PostProcessing.logit_scale', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _POSTPROCESSING_SCORECONVERTER,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=298,
  serialized_end=603,
)

_POSTPROCESSING.fields_by_name['batch_non_max_suppression'].message_type = _BATCHNONMAXSUPPRESSION
_POSTPROCESSING.fields_by_name['score_converter'].enum_type = _POSTPROCESSING_SCORECONVERTER
_POSTPROCESSING_SCORECONVERTER.containing_type = _POSTPROCESSING
DESCRIPTOR.message_types_by_name['BatchNonMaxSuppression'] = _BATCHNONMAXSUPPRESSION
DESCRIPTOR.message_types_by_name['PostProcessing'] = _POSTPROCESSING

BatchNonMaxSuppression = _reflection.GeneratedProtocolMessageType('BatchNonMaxSuppression', (_message.Message,), dict(
  DESCRIPTOR = _BATCHNONMAXSUPPRESSION,
  __module__ = 'rastervision.protos.tf_object_detection.post_processing_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.BatchNonMaxSuppression)
  ))
_sym_db.RegisterMessage(BatchNonMaxSuppression)

PostProcessing = _reflection.GeneratedProtocolMessageType('PostProcessing', (_message.Message,), dict(
  DESCRIPTOR = _POSTPROCESSING,
  __module__ = 'rastervision.protos.tf_object_detection.post_processing_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.PostProcessing)
  ))
_sym_db.RegisterMessage(PostProcessing)


# @@protoc_insertion_point(module_scope)



import os

from rastervision.backend.keras_classification.models.resnet50 import ResNet50
from rastervision.protos.keras_classification.model_pb2 import Model


def build_from_path(model_path):
    import keras

    return keras.models.load_model(model_path)


def build(model_options, pretrained_model_path):
    if os.path.isfile(model_options.model_path):
        return build_from_path(model_options.model_path)

    nb_channels = 3
    input_shape = (model_options.input_size, model_options.input_size,
                   nb_channels)
    activation = 'softmax'

    if model_options.type == Model.Type.Value('RESNET50'):
        model = ResNet50(
            include_top=True,
            weights=pretrained_model_path,
            load_weights_by_name=model_options.load_weights_by_name,
            input_shape=input_shape,
            classes=model_options.nb_classes,
            activation=activation)
    else:
        raise ValueError(
            Model.Type.Name(model_options.type) + ' is not a valid model_type')

    return model



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/anchor_generator.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from rastervision.protos.tf_object_detection import grid_anchor_generator_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_grid__anchor__generator__pb2
from rastervision.protos.tf_object_detection import ssd_anchor_generator_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_ssd__anchor__generator__pb2
from rastervision.protos.tf_object_detection import multiscale_anchor_generator_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_multiscale__anchor__generator__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/anchor_generator.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n>rastervision/protos/tf_object_detection/anchor_generator.proto\x12\'rastervision.protos.tf_object_detection\x1a\x43rastervision/protos/tf_object_detection/grid_anchor_generator.proto\x1a\x42rastervision/protos/tf_object_detection/ssd_anchor_generator.proto\x1aIrastervision/protos/tf_object_detection/multiscale_anchor_generator.proto\"\xd2\x02\n\x0f\x41nchorGenerator\x12]\n\x15grid_anchor_generator\x18\x01 \x01(\x0b\x32<.rastervision.protos.tf_object_detection.GridAnchorGeneratorH\x00\x12[\n\x14ssd_anchor_generator\x18\x02 \x01(\x0b\x32;.rastervision.protos.tf_object_detection.SsdAnchorGeneratorH\x00\x12i\n\x1bmultiscale_anchor_generator\x18\x03 \x01(\x0b\x32\x42.rastervision.protos.tf_object_detection.MultiscaleAnchorGeneratorH\x00\x42\x18\n\x16\x61nchor_generator_oneof')
  ,
  dependencies=[rastervision_dot_protos_dot_tf__object__detection_dot_grid__anchor__generator__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_ssd__anchor__generator__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_multiscale__anchor__generator__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_ANCHORGENERATOR = _descriptor.Descriptor(
  name='AnchorGenerator',
  full_name='rastervision.protos.tf_object_detection.AnchorGenerator',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grid_anchor_generator', full_name='rastervision.protos.tf_object_detection.AnchorGenerator.grid_anchor_generator', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ssd_anchor_generator', full_name='rastervision.protos.tf_object_detection.AnchorGenerator.ssd_anchor_generator', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='multiscale_anchor_generator', full_name='rastervision.protos.tf_object_detection.AnchorGenerator.multiscale_anchor_generator', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='anchor_generator_oneof', full_name='rastervision.protos.tf_object_detection.AnchorGenerator.anchor_generator_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=320,
  serialized_end=658,
)

_ANCHORGENERATOR.fields_by_name['grid_anchor_generator'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_grid__anchor__generator__pb2._GRIDANCHORGENERATOR
_ANCHORGENERATOR.fields_by_name['ssd_anchor_generator'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_ssd__anchor__generator__pb2._SSDANCHORGENERATOR
_ANCHORGENERATOR.fields_by_name['multiscale_anchor_generator'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_multiscale__anchor__generator__pb2._MULTISCALEANCHORGENERATOR
_ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof'].fields.append(
  _ANCHORGENERATOR.fields_by_name['grid_anchor_generator'])
_ANCHORGENERATOR.fields_by_name['grid_anchor_generator'].containing_oneof = _ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof']
_ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof'].fields.append(
  _ANCHORGENERATOR.fields_by_name['ssd_anchor_generator'])
_ANCHORGENERATOR.fields_by_name['ssd_anchor_generator'].containing_oneof = _ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof']
_ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof'].fields.append(
  _ANCHORGENERATOR.fields_by_name['multiscale_anchor_generator'])
_ANCHORGENERATOR.fields_by_name['multiscale_anchor_generator'].containing_oneof = _ANCHORGENERATOR.oneofs_by_name['anchor_generator_oneof']
DESCRIPTOR.message_types_by_name['AnchorGenerator'] = _ANCHORGENERATOR

AnchorGenerator = _reflection.GeneratedProtocolMessageType('AnchorGenerator', (_message.Message,), dict(
  DESCRIPTOR = _ANCHORGENERATOR,
  __module__ = 'rastervision.protos.tf_object_detection.anchor_generator_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.AnchorGenerator)
  ))
_sym_db.RegisterMessage(AnchorGenerator)


# @@protoc_insertion_point(module_scope)



# flake8: noqa

from rastervision.runner.command_definition import *
from rastervision.runner.command_dag import *
from rastervision.runner.experiment_runner import *
from rastervision.runner.out_of_process_experiment_runner import *
from rastervision.runner.inprocess_experiment_runner import *
from rastervision.runner.aws_batch_experiment_runner import *
from rastervision.runner.local_experiment_runner import *
from rastervision.runner.command_runner import *



import os
from copy import deepcopy
from google.protobuf import (text_format, json_format)

import rastervision as rv
from rastervision.backend import (BackendConfig, BackendConfigBuilder)
from rastervision.protos.backend_pb2 import BackendConfig as BackendConfigMsg
from rastervision.protos.tf_object_detection.pipeline_pb2 import TrainEvalPipelineConfig
from rastervision.utils.files import file_to_str
from rastervision.utils.misc import set_nested_keys
from rastervision.task import ObjectDetectionConfig

# Default location to Tensorflow Object Detection's scripts.
DEFAULT_SCRIPT_TRAIN = '/opt/tf-models/object_detection/model_main.py'
DEFAULT_SCRIPT_EXPORT = '/opt/tf-models/object_detection/export_inference_graph.py'
CHIP_OUTPUT_FILES = ['training', 'validation']
DEBUG_CHIP_OUTPUT_FILES = [
    'train-debug-chips.zip', 'validation-debug-chips.zip'
]


class TFObjectDetectionConfig(BackendConfig):
    class TrainOptions:
        def __init__(self,
                     sync_interval=600,
                     do_monitoring=True,
                     replace_model=False):
            self.sync_interval = sync_interval
            self.do_monitoring = do_monitoring
            self.replace_model = replace_model

    class ScriptLocations:
        def __init__(self,
                     model_main_uri=DEFAULT_SCRIPT_TRAIN,
                     export_uri=DEFAULT_SCRIPT_EXPORT):
            self.model_main_uri = model_main_uri
            self.export_uri = export_uri

    def __init__(self,
                 tfod_config,
                 pretrained_model_uri=None,
                 train_options=None,
                 script_locations=None,
                 debug=False,
                 training_data_uri=None,
                 training_output_uri=None,
                 model_uri=None,
                 fine_tune_checkpoint_name=None):
        if train_options is None:
            train_options = TFObjectDetectionConfig.TrainOptions()
        if script_locations is None:
            script_locations = TFObjectDetectionConfig.ScriptLocations()

        super().__init__(rv.TF_OBJECT_DETECTION, pretrained_model_uri)
        self.tfod_config = tfod_config
        self.pretrained_model_uri = pretrained_model_uri
        self.train_options = train_options
        self.script_locations = script_locations
        self.debug = debug

        # Internally set from command preprocessing
        self.training_data_uri = training_data_uri
        self.training_output_uri = training_output_uri
        self.model_uri = model_uri
        self.fine_tune_checkpoint_name = fine_tune_checkpoint_name

    def create_backend(self, task_config):
        from rastervision.backend.tf_object_detection import TFObjectDetection
        return TFObjectDetection(self, task_config)

    def get_num_steps(self):
        return int(self.tfod_config['trainConfig']['numSteps'])

    def get_batch_size(self, batch_size):
        return int(self.tfod_config.train_config['trainConfig']['batchSize'])

    def to_proto(self):
        d = {
            'sync_interval': self.train_options.sync_interval,
            'do_monitoring': self.train_options.do_monitoring,
            'replace_model': self.train_options.replace_model,
            'model_main_py': self.script_locations.model_main_uri,
            'export_py': self.script_locations.export_uri,
            'training_data_uri': self.training_data_uri,
            'training_output_uri': self.training_output_uri,
            'model_uri': self.model_uri,
            'debug': self.debug,
            'fine_tune_checkpoint_name': self.fine_tune_checkpoint_name,
            'tfod_config': self.tfod_config
        }

        conf = json_format.ParseDict(
            d, BackendConfigMsg.TFObjectDetectionConfig())

        msg = BackendConfigMsg(
            backend_type=rv.TF_OBJECT_DETECTION,
            tf_object_detection_config=conf)

        if self.pretrained_model_uri:
            msg.MergeFrom(
                BackendConfigMsg(
                    pretrained_model_uri=self.pretrained_model_uri))

        return msg

    def save_bundle_files(self, bundle_dir):
        if not self.model_uri:
            raise rv.ConfigError('model_uri is not set.')
        local_path, base_name = self.bundle_file(self.model_uri, bundle_dir)
        new_config = self.to_builder() \
                         .with_model_uri(base_name) \
                         .build()
        return (new_config, [local_path])

    def load_bundle_files(self, bundle_dir):
        if not self.model_uri:
            raise rv.ConfigError('model_uri is not set.')
        local_model_uri = os.path.join(bundle_dir, self.model_uri)
        return self.to_builder() \
                   .with_model_uri(local_model_uri) \
                   .build()

    def update_for_command(self, command_type, experiment_config,
                           context=None):
        super().update_for_command(command_type, experiment_config, context)
        if command_type == rv.CHIP:
            if not self.training_data_uri:
                self.training_data_uri = experiment_config.chip_uri

        if command_type == rv.TRAIN:
            if not self.training_output_uri:
                self.training_output_uri = experiment_config.train_uri
            if not self.model_uri:
                self.model_uri = os.path.join(self.training_output_uri,
                                              'model')
            if not self.fine_tune_checkpoint_name:
                # Set the fine tune checkpoint name to the experiment id
                self.fine_tune_checkpoint_name = experiment_config.id

    def report_io(self, command_type, io_def):
        super().report_io(command_type, io_def)
        if command_type == rv.CHIP:
            outputs = list(
                map(lambda x: os.path.join(self.training_data_uri, x),
                    CHIP_OUTPUT_FILES))
            if self.debug:
                outputs.extend(
                    list(
                        map(lambda x: os.path.join(self.training_data_uri, x),
                            DEBUG_CHIP_OUTPUT_FILES)))
            io_def.add_outputs(outputs)

        if command_type == rv.TRAIN:
            if not self.training_data_uri:
                io_def.add_missing('Missing training_data_uri.')
            else:
                inputs = list(
                    map(lambda x: os.path.join(self.training_data_uri, x),
                        CHIP_OUTPUT_FILES))
                io_def.add_inputs(inputs)

            io_def.add_output(self.model_uri)
            io_def.add_output(self.fine_tune_checkpoint_name)

        if command_type in [rv.PREDICT, rv.BUNDLE]:
            if not self.model_uri:
                io_def.add_missing('Missing model_uri.')
            else:
                io_def.add_input(self.model_uri)


class TFObjectDetectionConfigBuilder(BackendConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'tfod_config': prev.tfod_config,
                'pretrained_model_uri': prev.pretrained_model_uri,
                'train_options': prev.train_options,
                'script_locations': prev.script_locations,
                'debug': prev.debug,
                'training_data_uri': prev.training_data_uri,
                'training_output_uri': prev.training_output_uri,
                'model_uri': prev.model_uri,
                'fine_tune_checkpoint_name': prev.fine_tune_checkpoint_name
            }
        super().__init__(rv.TF_OBJECT_DETECTION, TFObjectDetectionConfig,
                         config, prev)
        self.config_mods = []
        self.require_task = prev is None

    def from_proto(self, msg):
        b = super().from_proto(msg)
        conf = msg.tf_object_detection_config
        # Since this is coming from a serialized message,
        # assume the task has already been set and do not
        # require it during validation.
        b.require_task = False
        b = b.with_train_options(
            sync_interval=conf.sync_interval,
            do_monitoring=conf.do_monitoring,
            replace_model=conf.replace_model)
        b = b.with_script_locations(
            model_main_uri=conf.model_main_py, export_uri=conf.export_py)
        b = b.with_training_data_uri(conf.training_data_uri)
        b = b.with_training_output_uri(conf.training_output_uri)
        b = b.with_model_uri(conf.model_uri)
        b = b.with_fine_tune_checkpoint_name(conf.fine_tune_checkpoint_name)
        b = b.with_debug(conf.debug)

        return b.with_template(json_format.MessageToDict(conf.tfod_config))

    def validate(self):
        super().validate()
        if not self.config.get('tfod_config'):
            raise rv.ConfigError('You must specify a template for the backend '
                                 "configuration - use 'with_template'.")
        if not isinstance(self.config.get('tfod_config'), dict):
            raise rv.ConfigError(
                'tfod_config must be of type dict, got {}'.format(
                    type(self.config.get('tfod_config'))))
        if self.require_task and not self.task:
            raise rv.ConfigError('You must specify the task this backend '
                                 "is for - use 'with_task'.")
        if self.require_task and not isinstance(self.task,
                                                ObjectDetectionConfig):
            raise rv.ConfigError(
                'Task set with with_task must be of type ObjectDetectionConfig, got {}.'.
                format(type(self.task)))
        return True

    def build(self):
        """Build this configuration.

        Set any values into the TF object detection pipeline config as
        necessary.
        """
        self.validate()

        b = deepcopy(self)

        # Check if a pretrained model was assigned.
        pretrained_model = b.config.get('pretrained_model_uri')
        if pretrained_model:
            b = b.with_config({'fineTuneCheckpoint': pretrained_model})
        else:
            b = b.with_config(
                {
                    'fineTuneCheckpoint': ''
                }, ignore_missing_keys=True)

        for config_mod, ignore_missing_keys, set_missing_keys in b.config_mods:
            try:
                set_nested_keys(b.config['tfod_config'], config_mod,
                                ignore_missing_keys, set_missing_keys)
            except Exception as e:
                raise rv.ConfigError(
                    'Error setting configuration {}'.format(config_mod)) from e

        return TFObjectDetectionConfig(**b.config)

    def _applicable_tasks(self):
        return [rv.OBJECT_DETECTION]

    def _process_task(self):
        return self.with_config(
            {
                'numClasses': len(self.task.class_map.get_items()),
                'imageResizer': {
                    'fixedShapeResizer': {
                        'height': self.task.chip_size,
                        'width': self.task.chip_size
                    },
                    'keepAspectRatioResizer': {
                        'minDimension': self.task.chip_size,
                        'maxDimension': self.task.chip_size
                    }
                }
            },
            ignore_missing_keys=True)

    def _load_model_defaults(self, model_defaults):
        """Loads defaults. Expected keys are "pretrained_model_uri" and "pipeline_config_uri",
           neither of which is required.
        """
        expected_keys = ['pretrained_model_uri', 'pipeline_config_uri']
        unknown_keys = set(model_defaults.keys()) - set(expected_keys)
        if unknown_keys:
            raise rv.ConfigError('Unexpected keys in model defaults:'
                                 ' {}. Expected keys: {}'.format(
                                     unknown_keys, expected_keys))

        b = self
        if 'pretrained_model_uri' in model_defaults:
            b = b.with_pretrained_model(model_defaults['pretrained_model_uri'])
        if 'pipeline_config_uri' in model_defaults:
            b = b.with_template(model_defaults['pipeline_config_uri'])
        return b

    def with_template(self, template):
        """Use a template for TF Object Detection pipeline config.

        Args:
            template: A dict, string or uri as the base for the TF
                Object Detection API model training pipeline, for example those
                found here:
                https://github.com/tensorflow/models/tree/eef6bb5bd3b3cd5fcf54306bf29750b7f9f9a5ea/research/object_detection/samples/configs # noqa
        """
        template_json = None
        if type(template) is dict:
            template_json = template
        else:
            # Try parsing the string as a message, on fail assume it's a URI
            msg = None
            try:
                msg = text_format.Parse(template, TrainEvalPipelineConfig())
            except text_format.ParseError:
                msg = text_format.Parse(
                    file_to_str(template), TrainEvalPipelineConfig())
            template_json = json_format.MessageToDict(msg)

        b = deepcopy(self)
        b.config['tfod_config'] = template_json
        return b

    def with_batch_size(self, batch_size):
        """Sets the training batch size."""
        return self.with_config(
            {
                'trainConfig': {
                    'batchSize': batch_size
                }
            }, set_missing_keys=True)

    def with_num_steps(self, num_steps):
        """Sets the number of training steps."""
        return self.with_config(
            {
                'trainConfig': {
                    'numSteps': num_steps
                }
            }, set_missing_keys=True)

    def with_config(self,
                    config_mod,
                    ignore_missing_keys=False,
                    set_missing_keys=False):
        """Given a dict, modify the tensorflow pipeline configuration.

        Modify it such that keys that are found recursively in the
        configuration are replaced with those values. TODO: better explanation.
        """
        b = deepcopy(self)
        b.config_mods.append((config_mod, ignore_missing_keys,
                              set_missing_keys))
        return b

    def with_debug(self, debug):
        """Sets the debug flag for this backend.
        """
        b = deepcopy(self)
        b.config['debug'] = debug
        return b

    def with_training_data_uri(self, training_data_uri):
        """Whence comes the training data?

        Args:
            training_data_uri: The location of the training data.
        """
        b = deepcopy(self)
        b.config['training_data_uri'] = training_data_uri
        return b

    def with_training_output_uri(self, training_output_uri):
        """Whither goes the training output?

        Args:
            training_output_uri: The location where the training
                output will be stored.
        """
        b = deepcopy(self)
        b.config['training_output_uri'] = training_output_uri
        return b

    def with_model_uri(self, model_uri):
        """Defines the name of the model file that will be created for
        this model after training.

        """
        b = deepcopy(self)
        b.config['model_uri'] = model_uri
        return b

    def with_fine_tune_checkpoint_name(self, fine_tune_checkpoint_name):
        """Defines the name of the fine tune checkpoint for this model."""
        b = deepcopy(self)
        b.config['fine_tune_checkpoint_name'] = fine_tune_checkpoint_name
        return b

    def with_train_options(self,
                           sync_interval=600,
                           do_monitoring=True,
                           replace_model=False):
        """Sets the train options for this backend.

       Args:
          sync_interval: How often to sync output of training
                         to the cloud (in seconds).

          do_monitoring: Run process to monitor training (eg. Tensorboard)

          replace_model: Replace the model checkpoint if exists.
                         If false, this will continue training from
                         checkpoing if exists, if the backend allows for this.
        """
        b = deepcopy(self)
        b.config['train_options'] = TFObjectDetectionConfig.TrainOptions(
            sync_interval, do_monitoring, replace_model)
        return b

    def with_script_locations(self,
                              model_main_uri=DEFAULT_SCRIPT_TRAIN,
                              export_uri=DEFAULT_SCRIPT_EXPORT):
        sl = TFObjectDetectionConfig.ScriptLocations(model_main_uri,
                                                     export_uri)
        b = deepcopy(self)
        b.config['script_locations'] = sl
        return b



from copy import deepcopy
from typing import (List, Dict, Tuple, Union)

import rastervision as rv
from rastervision.task import ChipClassification
from rastervision.core.class_map import (ClassMap, ClassItem)
from rastervision.task import (TaskConfig, TaskConfigBuilder)
from rastervision.protos.task_pb2 import TaskConfig as TaskConfigMsg
from rastervision.protos.class_item_pb2 import ClassItem as ClassItemMsg


class ChipClassificationConfig(TaskConfig):
    def __init__(self,
                 class_map,
                 predict_batch_size=10,
                 predict_package_uri=None,
                 debug=False,
                 predict_debug_uri=None,
                 chip_size=300):
        super().__init__(rv.CHIP_CLASSIFICATION, predict_batch_size,
                         predict_package_uri, debug, predict_debug_uri)
        self.class_map = class_map
        self.chip_size = chip_size

    def create_task(self, backend):
        return ChipClassification(self, backend)

    def to_proto(self):
        conf = TaskConfigMsg.ChipClassificationConfig(
            chip_size=self.chip_size, class_items=self.class_map.to_proto())
        return TaskConfigMsg(
            task_type=rv.CHIP_CLASSIFICATION,
            chip_classification_config=conf,
            predict_package_uri=self.predict_package_uri)

    def save_bundle_files(self, bundle_dir):
        return (self, [])

    def load_bundle_files(self, bundle_dir):
        return self


class ChipClassificationConfigBuilder(TaskConfigBuilder):
    def __init__(self, prev=None):
        config = {}
        if prev:
            config = {
                'class_map': prev.class_map,
                'chip_size': prev.chip_size,
                'predict_batch_size': prev.predict_batch_size,
                'predict_package_uri': prev.predict_package_uri,
                'debug': prev.debug,
                'predict_debug_uri': prev.predict_debug_uri
            }
        super().__init__(ChipClassificationConfig, config)

    def validate(self):
        if 'class_map' not in self.config:
            raise rv.ConfigError('Class map required for this task. '
                                 'Use "with_classes"')
        if not isinstance(self.config['class_map'], ClassMap):
            raise rv.ConfigError(
                'Class map set with "with_classes" must be of type ClassMap, got {}'.
                format(type(self.config['class_map'])))

    def from_proto(self, msg):
        b = super().from_proto(msg)
        conf = msg.chip_classification_config
        return b.with_classes(list(conf.class_items)) \
                .with_predict_package_uri(msg.predict_package_uri) \
                .with_chip_size(conf.chip_size)

    def with_classes(
            self, classes: Union[ClassMap, List[str], List[ClassItemMsg], List[
                ClassItem], Dict[str, int], Dict[str, Tuple[int, str]]]):
        """Set the classes for this task.

            Args:
                classes: Either a list of class names, a dict which
                         maps class names to class ids, or a dict
                         which maps class names to a tuple of (class_id, color),
                         where color is a PIL color string.
        """
        b = deepcopy(self)
        b.config['class_map'] = ClassMap.construct_from(classes)
        return b

    def with_chip_size(self, chip_size):
        """Set the chip_size for this task.

        Note that some model implementations have a minimum size of input they
        can handle. A value of > 200 is usually safe.

        Args:
            chip_size: (int) chip size in units of pixels
        """
        b = deepcopy(self)
        b.config['chip_size'] = chip_size
        return b



import click

from rastervision.backend.keras_classification.utils \
    import load_json_config
from rastervision.backend.keras_classification.builders \
    import trainer_builder, model_builder
from rastervision.protos.keras_classification.pipeline_pb2 \
    import PipelineConfig


def _train(config_path, pretrained_model_path, do_monitoring):
    config = load_json_config(config_path, PipelineConfig())
    model = model_builder.build(config.model, pretrained_model_path)
    trainer = trainer_builder.build(config.trainer, model)
    trainer.train(do_monitoring)


@click.command()
@click.argument('config_path')
def train(config_path):
    _train(config_path)


if __name__ == '__main__':
    train()



import numpy as np

from rastervision.evaluation import EvaluationItem


class ClassEvaluationItem(EvaluationItem):
    """Evaluation metrics for a single class (or average over classes).

    None is used for values that are undefined because they involve a division
    by zero (eg. precision when there are no predictions).
    """

    def __init__(self,
                 precision=None,
                 recall=None,
                 f1=None,
                 count_error=None,
                 gt_count=0,
                 class_id=None,
                 class_name=None,
                 conf_mat=None):
        self.precision = precision
        self.recall = recall
        self.f1 = f1
        self.count_error = count_error
        # Ground truth count of elements (boxes for object detection, pixels
        # for segmentation, cells for classification).
        self.gt_count = gt_count
        self.conf_mat = conf_mat
        self.class_id = class_id
        self.class_name = class_name

    def merge(self, other):
        """Merges another item from a different scene into this one.

        This is used to average metrics over scenes. Merges by taking a
        weighted average (by gt_count) of the metrics.
        """
        if other.gt_count > 0:
            total_gt_count = self.gt_count + other.gt_count
            self_ratio = self.gt_count / total_gt_count
            other_ratio = other.gt_count / total_gt_count

            def weighted_avg(self_val, other_val):
                if self_val is None and other_val is None:
                    return 0.0
                # Handle a single None value by setting them to zero.
                return (self_ratio * (self_val or 0) +
                        other_ratio * (other_val or 0))

            self.precision = weighted_avg(self.precision, other.precision)
            self.recall = weighted_avg(self.recall, other.recall)
            self.f1 = weighted_avg(self.f1, other.f1)
            self.count_error = weighted_avg(self.count_error,
                                            other.count_error)
            self.gt_count = total_gt_count

        if other.conf_mat is not None:
            if self.class_name == 'average':
                if self.conf_mat is None:
                    # Make first row all zeros so that the array indices
                    # correspond to valid class ids (ie. >= 1).
                    self.conf_mat = np.concatenate(
                        [
                            np.zeros_like(other.conf_mat)[np.newaxis, :],
                            np.array(other.conf_mat)[np.newaxis, :]
                        ],
                        axis=0)
                else:
                    self.conf_mat = np.concatenate(
                        [self.conf_mat, other.conf_mat[np.newaxis, :]], axis=0)
            else:
                self.conf_mat += other.conf_mat

    def to_json(self):
        new_dict = {}
        for k, v in self.__dict__.items():
            new_dict[k] = v.tolist() if isinstance(v, np.ndarray) else v
        if new_dict['conf_mat'] is None:
            del new_dict['conf_mat']
        return new_dict

    def __repr__(self):
        return str(self.to_json())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/eval.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/eval.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n2rastervision/protos/tf_object_detection/eval.proto\x12\'rastervision.protos.tf_object_detection\"\xf7\x05\n\nEvalConfig\x12\x15\n\nbatch_size\x18\x19 \x01(\r:\x01\x31\x12\x1e\n\x12num_visualizations\x18\x01 \x01(\r:\x02\x31\x30\x12\x1e\n\x0cnum_examples\x18\x02 \x01(\r:\x04\x35\x30\x30\x30\x42\x02\x18\x01\x12\x1f\n\x12\x65val_interval_secs\x18\x03 \x01(\r:\x03\x33\x30\x30\x12\x18\n\tmax_evals\x18\x04 \x01(\r:\x01\x30\x42\x02\x18\x01\x12\x19\n\nsave_graph\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\"\n\x18visualization_export_dir\x18\x06 \x01(\t:\x00\x12\x15\n\x0b\x65val_master\x18\x07 \x01(\t:\x00\x12\x13\n\x0bmetrics_set\x18\x08 \x03(\t\x12\x15\n\x0b\x65xport_path\x18\t \x01(\t:\x00\x12!\n\x12ignore_groundtruth\x18\n \x01(\x08:\x05\x66\x61lse\x12\"\n\x13use_moving_averages\x18\x0b \x01(\x08:\x05\x66\x61lse\x12\"\n\x13\x65val_instance_masks\x18\x0c \x01(\x08:\x05\x66\x61lse\x12 \n\x13min_score_threshold\x18\r \x01(\x02:\x03\x30.5\x12&\n\x1amax_num_boxes_to_visualize\x18\x0e \x01(\x05:\x02\x32\x30\x12\x1a\n\x0bskip_scores\x18\x0f \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x0bskip_labels\x18\x10 \x01(\x08:\x05\x66\x61lse\x12*\n\x1bvisualize_groundtruth_boxes\x18\x11 \x01(\x08:\x05\x66\x61lse\x12\x32\n#groundtruth_box_visualization_color\x18\x12 \x01(\t:\x05\x62lack\x12\x35\n&keep_image_id_for_visualization_export\x18\x13 \x01(\x08:\x05\x66\x61lse\x12$\n\x16retain_original_images\x18\x17 \x01(\x08:\x04true\x12+\n\x1cinclude_metrics_per_category\x18\x18 \x01(\x08:\x05\x66\x61lse')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_EVALCONFIG = _descriptor.Descriptor(
  name='EvalConfig',
  full_name='rastervision.protos.tf_object_detection.EvalConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='rastervision.protos.tf_object_detection.EvalConfig.batch_size', index=0,
      number=25, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_visualizations', full_name='rastervision.protos.tf_object_detection.EvalConfig.num_visualizations', index=1,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=10,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_examples', full_name='rastervision.protos.tf_object_detection.EvalConfig.num_examples', index=2,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=5000,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='eval_interval_secs', full_name='rastervision.protos.tf_object_detection.EvalConfig.eval_interval_secs', index=3,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=300,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_evals', full_name='rastervision.protos.tf_object_detection.EvalConfig.max_evals', index=4,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='save_graph', full_name='rastervision.protos.tf_object_detection.EvalConfig.save_graph', index=5,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='visualization_export_dir', full_name='rastervision.protos.tf_object_detection.EvalConfig.visualization_export_dir', index=6,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eval_master', full_name='rastervision.protos.tf_object_detection.EvalConfig.eval_master', index=7,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metrics_set', full_name='rastervision.protos.tf_object_detection.EvalConfig.metrics_set', index=8,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='export_path', full_name='rastervision.protos.tf_object_detection.EvalConfig.export_path', index=9,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ignore_groundtruth', full_name='rastervision.protos.tf_object_detection.EvalConfig.ignore_groundtruth', index=10,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_moving_averages', full_name='rastervision.protos.tf_object_detection.EvalConfig.use_moving_averages', index=11,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eval_instance_masks', full_name='rastervision.protos.tf_object_detection.EvalConfig.eval_instance_masks', index=12,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='min_score_threshold', full_name='rastervision.protos.tf_object_detection.EvalConfig.min_score_threshold', index=13,
      number=13, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.5),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_num_boxes_to_visualize', full_name='rastervision.protos.tf_object_detection.EvalConfig.max_num_boxes_to_visualize', index=14,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=20,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='skip_scores', full_name='rastervision.protos.tf_object_detection.EvalConfig.skip_scores', index=15,
      number=15, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='skip_labels', full_name='rastervision.protos.tf_object_detection.EvalConfig.skip_labels', index=16,
      number=16, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='visualize_groundtruth_boxes', full_name='rastervision.protos.tf_object_detection.EvalConfig.visualize_groundtruth_boxes', index=17,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='groundtruth_box_visualization_color', full_name='rastervision.protos.tf_object_detection.EvalConfig.groundtruth_box_visualization_color', index=18,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=_b("black").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='keep_image_id_for_visualization_export', full_name='rastervision.protos.tf_object_detection.EvalConfig.keep_image_id_for_visualization_export', index=19,
      number=19, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='retain_original_images', full_name='rastervision.protos.tf_object_detection.EvalConfig.retain_original_images', index=20,
      number=23, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='include_metrics_per_category', full_name='rastervision.protos.tf_object_detection.EvalConfig.include_metrics_per_category', index=21,
      number=24, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=96,
  serialized_end=855,
)

DESCRIPTOR.message_types_by_name['EvalConfig'] = _EVALCONFIG

EvalConfig = _reflection.GeneratedProtocolMessageType('EvalConfig', (_message.Message,), dict(
  DESCRIPTOR = _EVALCONFIG,
  __module__ = 'rastervision.protos.tf_object_detection.eval_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.EvalConfig)
  ))
_sym_db.RegisterMessage(EvalConfig)


_EVALCONFIG.fields_by_name['num_examples'].has_options = True
_EVALCONFIG.fields_by_name['num_examples']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_EVALCONFIG.fields_by_name['max_evals'].has_options = True
_EVALCONFIG.fields_by_name['max_evals']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)
